# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from borg.localrole.interfaces import ILocalRoleProvider
from zope.interface import implements
from plone import api

from cpskin.citizen import utils
from cpskin.citizen.dashboard.interfaces import IAdminDashboard


class DashboardLocalRoleAdapter(object):
    implements(ILocalRoleProvider)

    def __init__(self, context):
        self.context = context
        current_user = api.user.get_current()
        self.is_citizen = utils.is_citizen(current_user)

    def getRoles(self, principal):
        """Grant permission for principal"""
        if IAdminDashboard.providedBy(self.context) or not self.is_citizen:
            return []
        return ("Reader",)

    def getAllRoles(self):
        """Grant permissions"""
        if IAdminDashboard.providedBy(self.context) or not self.is_citizen:
            yield ("", ("",))
            raise StopIteration
        yield ("Citizens", "Reader")



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from Acquisition import aq_inner
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from imio.actionspanel.browser.views import ActionsPanelView
from plone import api
from plone.app.stagingbehavior.utils import get_working_copy

from cpskin.citizen import utils


class ActionsView(ActionsPanelView):
    def __call__(self, *args, **kwargs):
        self.SECTIONS_TO_RENDER = ("render_citizen_actions",)
        self.working_copy = get_working_copy(aq_inner(self.context))
        return super(ActionsView, self).__call__(*args, **kwargs)

    def render_citizen_actions(self):
        """Render citizen actions"""
        return ViewPageTemplateFile("templates/actions_panel_citizen.pt")(self)

    @property
    def can_edit(self):
        return utils.can_edit_citizen(api.user.get_current(), self.context)

    @property
    def has_draft(self):
        return self.working_copy is not None

    @property
    def draft_url(self):
        return self.working_copy.absolute_url()



# -*- coding: utf-8 -*-

from Acquisition import aq_base
from Acquisition import aq_inner
from Products.Five import BrowserView
from plone.app.iterate.interfaces import IWorkingCopy
from plone.app.stagingbehavior.utils import get_relations
from zc.relation.interfaces import ICatalog
from zope.component import getUtility
from zope.interface import noLongerProvides


class CleanupView(BrowserView):
    def __call__(self, *args, **kwargs):
        context = aq_base(aq_inner(self.context))
        relations = get_relations(context)
        ref_catalog = getUtility(ICatalog)
        for relation in relations:
            if relation.from_object is None:
                # Remove the relation from the reference catalog
                ref_catalog.unindex(relation)
                # Remove the interface
                if IWorkingCopy.providedBy(context):
                    noLongerProvides(context, IWorkingCopy)
        self.request.response.redirect(self.context.absolute_url())



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from plone import api

from cpskin.citizen import utils


class CitizenCreationFolderAdapter(object):
    def __init__(self, context, request):
        self.context = context
        self.request = request

    def get_folder(self, navigation_root, portal_type):
        settings = utils.get_settings()
        path = settings.proposal_folders.get(portal_type, "")
        if path.startswith("/"):
            path = path[1:]
        try:
            return navigation_root.restrictedTraverse(str(path))
        except KeyError:
            raise KeyError("unknown creation folder: {0}".format(str(path)))


class CitizenDraftFolderAdapter(object):
    def __init__(self, context):
        self.context = context

    def get_folder(self):
        navigation_root = api.portal.get_navigation_root(self.context)
        return navigation_root["citizen-drafts"]


class CitizenDraftOrganizationAdapter(CitizenDraftFolderAdapter):
    def get_folder(self):
        navigation_root = api.portal.get_navigation_root(self.context)
        return navigation_root["annuaire"]



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from cpskin.citizen.testing import CPSKIN_CITIZEN_INTEGRATION_TESTING
from plone import api

import unittest


class TestSetup(unittest.TestCase):
    layer = CPSKIN_CITIZEN_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        self.installer = api.portal.get_tool("portal_quickinstaller")

    def test_product_installed(self):
        """Test if cpskin.citizen is installed."""
        self.assertTrue(self.installer.isProductInstalled("cpskin.citizen"))

    def test_browserlayer(self):
        """Test that ICpskinCitizenLayer is registered."""
        from cpskin.citizen.interfaces import ICpskinCitizenLayer
        from plone.browserlayer import utils

        self.assertIn(ICpskinCitizenLayer, utils.registered_layers())


class TestUninstall(unittest.TestCase):
    layer = CPSKIN_CITIZEN_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.installer = api.portal.get_tool("portal_quickinstaller")
        self.installer.uninstallProducts(["cpskin.citizen"])

    def test_product_uninstalled(self):
        """Test if cpskin.citizen is cleanly uninstalled."""
        self.assertFalse(self.installer.isProductInstalled("cpskin.citizen"))



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.eeafaceted.z3ctable.browser.views import FacetedTableView

from cpskin.citizen.dashboard import table


class UserContentTableView(table.CitizenContentTable, FacetedTableView):
    pass


class UserClaimTableView(table.CitizenClaimTable, FacetedTableView):
    pass


class UserMapTableView(table.CitizenMapTable, FacetedTableView):
    pass


class AdminContentTableView(table.AdminContentTable, FacetedTableView):
    pass


class AdminClaimTableView(table.AdminClaimTable, FacetedTableView):
    pass



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from plone.app.iterate.browser import diff
from plone.app.iterate.interfaces import IWorkingCopy, IBaseline
from plone.app.stagingbehavior import STAGING_RELATION_NAME
from zc.relation.interfaces import ICatalog
from zope.component import queryUtility
from zope.intid.interfaces import IIntIds


class DiffView(diff.DiffView):
    def __init__(self, context, request):
        self.context = context
        self.request = request
        # if context.portal_type == 'Event':
        #     if not getattr(context, 'location', False):
        #         context.location = None
        catalog = queryUtility(ICatalog)
        intids = queryUtility(IIntIds)
        if IBaseline.providedBy(self.context):
            self.baseline = context
            self.working_copy = (
                catalog.findRelations(
                    dict(
                        from_id=intids.getId(context),
                        from_attribute=STAGING_RELATION_NAME,
                    )
                )
                .next()
                .to_object
            )
        elif IWorkingCopy.providedBy(self.context):
            self.working_copy = context
            self.baseline = (
                catalog.findRelations(
                    dict(
                        to_id=intids.getId(context),
                        from_attribute=STAGING_RELATION_NAME,
                    )
                )
                .next()
                .from_object
            )
        else:
            raise AttributeError("Invalid Context")



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer


class ICpskinCitizenLayer(IDefaultBrowserLayer):
    """Marker interface that defines a browser layer."""


class ICitizenContentSubMenu(Interface):
    pass


class ICitizenProposeContentFolder(Interface):
    pass


class ICitizenCreationFolder(Interface):
    pass


class ICitizenDraftFolder(Interface):
    pass


class ICitizenDashboardFolder(Interface):
    pass


class IFacetedConfig(Interface):
    pass



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from Products.Five import BrowserView
from cpskin.citizen import notification
from cpskin.citizen import utils
from plone.app.iterate.browser.checkin import Checkin


class ReturnView(BrowserView):
    def __call__(self):
        view_url = self.context.absolute_url()
        form = self.request.form
        if "form.button.Confirm" in form:
            annotations = utils.get_annotations(self.context)
            annotations["validation_required"] = False
            annotations["comment"] = form.get("return_message", u"")
            self.context.reindexObject()
            notification.notify_content_refused(self.context)
            self.request.response.redirect(view_url)
        elif "form.button.Cancel" in form:
            self.request.response.redirect(view_url)
        return self.index()


class CheckinView(Checkin):
    def __call__(self):
        result = super(CheckinView, self).__call__()
        notification.notify_content_validated(self.context)
        return result



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.eeafaceted.z3ctable.columns import BaseColumn
from collective.eeafaceted.z3ctable.columns import BaseColumnHeader
from imio.dashboard import columns
from imio.prettylink.interfaces import IPrettyLink
from plone import api
from zope.component import getUtility
from zope.i18n import translate
from zope.schema.interfaces import IVocabularyFactory

from cpskin.citizen import _
from cpskin.citizen import utils


class BaseCitizenColumn(BaseColumn):
    def _translate(self, msgid):
        return translate(msgid, context=self.request)


class DashboardColumnHeader(BaseColumnHeader):
    @property
    def faceted_url(self):
        return "/".join(self.request.get("URL").split("/")[:-1])


class CitizenDraftColumn(columns.PrettyLinkColumn, BaseCitizenColumn):
    header = _(u"Title")
    weight = 5
    params = {"target": "_blank"}

    def renderCell(self, item):
        obj = self._getObject(item)
        working_copy = utils.get_working_copy(obj)
        if working_copy:
            return self.getPrettyLink(working_copy)
        return self.getPrettyLink(obj)


class CitizenTitleColumn(columns.PrettyLinkColumn, BaseCitizenColumn):
    header = _(u"Title")
    weight = 5
    params = {"target": "_blank"}


class DraftStateColumn(BaseCitizenColumn):
    header = _(u"Modification state")
    weight = 10

    def renderCell(self, item):
        obj = self._getObject(item)
        working_copy = utils.get_working_copy(obj)
        if working_copy:
            annotations = utils.get_annotations(working_copy)
            if annotations.get("validation_required", False):
                return self._translate(_(u"Awaiting for validation"))
            if annotations.get("comment", None):
                return self._translate(_(u"Awaiting for changes"))
        else:
            return self._translate(_(u"None"))
        return self._translate(_(u"Draft"))


class CitizenStateColumn(BaseCitizenColumn):
    header = _(u"State")
    weight = 20

    @property
    def published_states(self):
        if not hasattr(self, "_published_states"):
            settings = utils.get_settings()
            self.table._published_states = settings.published_states
        return self.table._published_states

    def _render_state(self, state, help_msg):
        return u'{state} <span class="wf-info" title="{help_msg}">?</span>'.format(
            state=self._translate(state), help_msg=self._translate(help_msg)
        )

    def renderCell(self, item):
        obj = self._getObject(item)
        working_copy = utils.get_working_copy(obj)
        if working_copy:
            annotations = utils.get_annotations(working_copy)
            if annotations.get("validation_required", False):
                return self._render_state(
                    _(u"Awaiting for validation"),
                    _("The draft was submitted to administrator for approval"),
                )
            if annotations.get("comment", None):
                # XXX doit-on refléter que des changements sont attendus ?
                return self._render_state(
                    _(u"Awaiting for validation"),
                    _("The draft was submitted to administrator for approval"),
                )
        else:
            current_state = api.content.get_state(obj)
            if current_state in self.published_states:
                return self._render_state(
                    _(u"Published"),
                    _("The document is published and visible to visitors"),
                )
        return self._render_state(
            _(u"Draft"),
            _(
                "You are working on a new version of the document that are not "
                "yet published"
            ),
        )


class OnlineColumn(columns.PrettyLinkColumn, BaseCitizenColumn):
    header = _(u"Online version")
    weight = 90
    params = {"target": "_blank", "showLockedIcon": False, "showIcons": False}

    def getPrettyLink(self, obj):
        pl = IPrettyLink(obj)
        for k, v in self.params.items():
            setattr(pl, k, v)
        pl.notViewableHelpMessage = ""
        return pl.getLink()

    @property
    def published_states(self):
        if not hasattr(self, "_published_states"):
            settings = utils.get_settings()
            self.table._published_states = settings.published_states
        return self.table._published_states

    def renderCell(self, item):
        obj = self._getObject(item)
        try:
            online_obj = utils.get_baseline(obj)
        except KeyError:
            # This append when the original document was deleted
            return self._translate(_(u"Removed"))
        if not online_obj:
            # special case where content was assigned by admin without edit
            online_obj = obj
        current_state = api.content.get_state(online_obj)
        if current_state in self.published_states:
            self.params["isViewable"] = True
            self.params["contentValue"] = self._translate(_(u"Yes"))
        else:
            self.params["isViewable"] = False
            self.params["contentValue"] = self._translate(_(u"No"))
        return self.getPrettyLink(online_obj)


class CitizenUsersColumn(BaseCitizenColumn):
    header = _(u"Citizen Users")
    weight = 20

    def renderCell(self, item):
        obj = self._getObject(item)
        existing_users = self._get_existing_users(obj)
        claimed_users = self._get_claimed_users(obj)
        if not existing_users:
            return claimed_users
        if not claimed_users:
            return existing_users
        return u"{claimed_users} ({existing_users})".format(
            existing_users=existing_users, claimed_users=claimed_users
        )

    def _get_existing_users(self, obj):
        if getattr(obj, "citizens", None):
            return u", ".join(obj.citizens)

    def _get_claimed_users(self, obj):
        claims = []
        annotations = utils.get_annotations(obj)
        for claim in annotations.get("claim", []):
            user = api.user.get(userid=claim)
            if user:
                claims.append(user.getProperty("fullname"))
        claims.sort()
        return u", ".join(
            [u"<span class='claim'>{}</span>".format(e.decode("utf8")) for e in claims]
        )


class ActionsColumn(BaseCitizenColumn):
    header = _(u"Actions")
    weight = 100

    def _get_vocabulary_value(self, item):
        if not hasattr(self, "_vocabulary"):
            factory = getUtility(IVocabularyFactory, "cpskin.citizen.actions")
            self._vocabulary = factory(item)
        if not item.citizen_action:
            return self._translate(_(u"None"))
        return self._translate(self._vocabulary.getTerm(item.citizen_action).title)

    def renderCell(self, item):
        return self._get_vocabulary_value(item)



# -*- coding: utf-8 -*-
from cpskin.citizen import _
from plone.app.users.browser.personalpreferences import UserDataPanelAdapter
from plone.app.users.userdataschema import IUserDataSchema
from plone.app.users.userdataschema import IUserDataSchemaProvider
from zope.interface import implements
from zope import schema


class UserDataSchemaProvider(object):
    implements(IUserDataSchemaProvider)

    def getSchema(self):
        """
        """
        return IEnhancedUserDataSchema


class IEnhancedUserDataSchema(IUserDataSchema):
    """ Use all the fields from the default user data schema, and add various
    extra fields.
    """

    street = schema.TextLine(
        title=_(u"Street", default=u"Rue"),
        description=_(
            u"help_street_description",
            default=u"Indiquez le nom de votre rue, cette adresse sera utilisée dans le site pour géolocaliser ce qu'il se passe autour de chez vous.",
        ),
        required=False,
    )

    number = schema.TextLine(title=_(u"Number", default=u"Numéro"), required=False)

    zip_code = schema.TextLine(
        title=_(u"Post Code", default=u"Code postal"), required=False
    )

    location = schema.TextLine(title=_(u"City", default=u"Ville"), required=False)

    latitude = schema.TextLine(
        title=_(u"Latitude", default=u"Latitude"),
        description=_(
            u"help_latitude_coordinates",
            default=u"Coordinates (latitude) of your living place",
        ),
        required=False,
    )

    longitude = schema.TextLine(
        title=_(u"Longitude", default=u"Longitude"),
        description=_(
            u"help_longitude_coordinates",
            default=u"Coordinates (longitude) of your living place",
        ),
        required=False,
    )


class EnhancedUserDataPanelAdapter(UserDataPanelAdapter):
    """
    """

    def _set_property(self, key, value):
        return self.context.setMemberProperties(
            {key: value and value.encode("utf-8") or value}
        )

    def _get_property(self, key):
        return self.context.getProperty(key, "").decode("utf-8")

    @property
    def street(self):
        return self._get_property("street")

    @street.setter
    def street(self, value):
        return self._set_property("street", value)

    @property
    def number(self):
        return self._get_property("number")

    @number.setter
    def number(self, value):
        return self._set_property("number", value)

    @property
    def zip_code(self):
        return self._get_property("zip_code")

    @zip_code.setter
    def zip_code(self, value):
        return self._set_property("zip_code", value)

    @property
    def location(self):
        return self._get_property("location")

    @location.setter
    def location(self, value):
        return self._set_property("location", value)

    @property
    def latitude(self):
        return self._get_property("latitude")

    @latitude.setter
    def latitude(self, value):
        return self._set_property("latitude", value)

    @property
    def longitude(self):
        return self._get_property("longitude")

    @longitude.setter
    def longitude(self, value):
        return self._set_property("longitude", value)



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from zope.interface import Interface
from eea.facetednavigation.interfaces import IFacetedNavigable


class ICitizenDashboard(IFacetedNavigable):
    """Marker interface for dashboard folder"""


class IAdminDashboard(ICitizenDashboard):
    """Marker interface for admin dashboard folder"""


class ICitizenMyContent(Interface):
    """Marker interface for "My contents" dashboard"""


class ICitizenDashboardQuery(Interface):
    """Utility for citizen dashboard query extra filters"""


class IFacetedDashboardTable(Interface):
    """Marker interface for dashboard table"""


class IFacetedDashboardCitizenContentTable(IFacetedDashboardTable):
    """Citizen faceted content table"""


class IFacetedDashboardCitizenClaimTable(IFacetedDashboardTable):
    """Citizen faceted claim table"""


class IFacetedDashboardCitizenMapTable(IFacetedDashboardTable):
    """Citizen faceted claim table"""


class IFacetedDashboardAdminContentTable(IFacetedDashboardTable):
    """Admin faceted content table"""


class IFacetedDashboardAdminClaimTable(IFacetedDashboardTable):
    """Admin faceted claim table"""



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from Acquisition import aq_inner
from Acquisition import aq_parent
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone import api
from plone.app.portlets.portlets import base
from plone.portlets.interfaces import IPortletDataProvider
from zope.interface import implements
from zope.security import checkPermission

from cpskin.citizen import utils
from cpskin.citizen.interfaces import ICitizenContentSubMenu
from cpskin.citizen.interfaces import ICitizenDashboardFolder
from cpskin.citizen.dashboard.interfaces import IAdminDashboard


class IDashboardPortlet(IPortletDataProvider):
    """Interface for the dashboard portlet"""


class DashboardPortletAssignment(base.Assignment):
    implements(IDashboardPortlet)

    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)

    @property
    def title(self):
        """Return the title of the portlet in "manage portlets" screen"""
        return "citizen dashboard"


class DashboardPortletRenderer(base.Renderer):
    render = ViewPageTemplateFile("templates/dashboard-portlet.pt")

    def update(self, *args, **kwargs):
        super(DashboardPortletRenderer, self).update(*args, **kwargs)
        self.items = self.get_items()

    @property
    def can_view(self):
        return len(self.items) > 0

    @property
    def base_folder(self):
        context = aq_inner(self.context)
        while not ICitizenDashboardFolder.providedBy(context):
            context = aq_parent(context)
        return context

    def is_citizen(self):
        current_user = api.user.get_current()
        return utils.is_citizen(current_user)

    def is_sub_menu(self, obj):
        return ICitizenContentSubMenu.providedBy(obj)

    def get_admin_dashboards(self):
        return [
            d
            for i, d in self.base_folder.contentItems()
            if IAdminDashboard.providedBy(d) and checkPermission("zope2.View", d)
        ]

    def get_citizen_items(self):
        items = []
        for i, d in self.base_folder.contentItems():
            if IAdminDashboard.providedBy(d):
                continue
            if not checkPermission("zope2.View", d):
                continue
            items.append(d)
        return items

    def get_items(self):
        if api.user.is_anonymous():
            return []
        if self.is_citizen():
            return self.get_citizen_items()
        else:
            return self.get_admin_dashboards()

    def css_class(self, item):
        base = ""
        if self.is_sub_menu(item):
            base = "citizen-content-submenu"
        return "{0} {1}".format(base, item.id)


class DashboardPortletAddForm(base.NullAddForm):
    def create(self):
        return DashboardPortletAssignment()



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from Products.statusmessages.interfaces import IStatusMessage
from plone import api
from plone.app.stagingbehavior.utils import get_baseline
from plone.app.stagingbehavior.utils import get_working_copy
from plone.dexterity.browser.edit import DefaultEditForm
from plone.dexterity.events import EditFinishedEvent
from plone.dexterity.i18n import MessageFactory as DX_MF
from plone.dexterity.interfaces import IDexterityEditForm
from plone.z3cform import layout
from z3c.form import button
from zope.event import notify
from zope.interface import classImplements

from cpskin.citizen import utils


class EditCitizenForm(DefaultEditForm):
    _allowed_fieldsets = ["address", "contact_details", "schedule", "images"]

    @button.buttonAndHandler(DX_MF(u"Save"), name="save")
    def handleApply(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        current_user = api.user.get_current()
        if utils.can_edit(current_user, self.context):
            self.applyChanges(data)
        else:
            utils.execute_under_unrestricted_user(
                api.portal.get(), self.applyChanges, current_user.id, data
            )
        IStatusMessage(self.request).addStatusMessage(
            DX_MF(u"Changes saved"), "info success"
        )
        self.request.response.redirect(self.nextURL())
        notify(EditFinishedEvent(self.context))

    def update_groups(self):
        self.groups = tuple(
            [g for g in self.groups if g.__name__ in self._allowed_fieldsets]
        )

    def update(self):
        super(EditCitizenForm, self).update()
        self.update_groups()

    def extractData(self):
        self.update_groups()
        return super(EditCitizenForm, self).extractData()


DefaultEditView = layout.wrap_form(EditCitizenForm)
classImplements(DefaultEditView, IDexterityEditForm)


class EditCitizenView(DefaultEditView):
    def update(self):
        original = get_baseline(self.context)
        working_copy = get_working_copy(original)
        if original is None or self.context == original:
            url = "{0}/@@content-checkout".format(self.context.absolute_url())
            if working_copy is not None:
                url = "{0}/@@edit-citizen".format(working_copy.absolute_url())
            self.request.response.redirect(url)
            return
        super(EditCitizenView, self).update()



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from plone import api
from plone.app.testing import login
from plone.app.testing import logout

from cpskin.citizen import testing


class TestLocalRole(testing.BaseTestCase):
    layer = testing.CPSKIN_CITIZEN_INTEGRATION_TESTING

    def test_role_on_admin_dashboard(self):
        """Test the local roles on an admin dashboard"""
        manager = api.user.get("manager")
        roles = api.user.get_roles(user=manager, obj=self.dashboards["admin-content"])
        self.assertNotIn("Reader", roles)

        login(self.portal, "citizen")
        citizen = api.user.get("citizen")
        roles = api.user.get_roles(user=citizen, obj=self.dashboards["admin-content"])
        self.assertNotIn("Reader", roles)
        logout()

    def test_role_on_citizen_dashboard(self):
        """Test the local roles on a citizen dashboard"""
        manager = api.user.get("manager")
        roles = api.user.get_roles(user=manager, obj=self.dashboards["citizen-content"])
        self.assertNotIn("Reader", roles)

        login(self.portal, "citizen")
        citizen = api.user.get("citizen")
        roles = api.user.get_roles(user=citizen, obj=self.dashboards["citizen-content"])
        self.assertIn("Reader", roles)
        logout()



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from AccessControl import getSecurityManager
from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.SecurityManagement import setSecurityManager
from AccessControl.User import UnrestrictedUser as BaseUnrestrictedUser
from persistent.dict import PersistentDict
from plone import api
from plone.app.linkintegrity.exceptions import LinkIntegrityNotificationException
from plone.app.stagingbehavior.utils import get_baseline  # noqa API
from plone.app.stagingbehavior.utils import get_relations  # noqa API
from plone.app.stagingbehavior.utils import get_working_copy as get_wc
from plone.dexterity import utils
from plone.registry.interfaces import IRegistry
from z3c.form import field
from zc.relation.interfaces import ICatalog
from zope.annotation import IAnnotations
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.schema.vocabulary import SimpleVocabulary

from cpskin.citizen import ANNOTATION_KEY
from cpskin.citizen.behavior import ICitizenAccess
from cpskin.citizen.browser.settings import ISettings
from cpskin.citizen.interfaces import ICitizenCreationFolder


DRAFT_FOLDER_ID = "citizen-drafts"
DASHBOARD_FOLDER_ID = "citizen-dashboard"
DASHBOARD_FOLDERS_IDS = (
    "citizen-content",
    "citizen-claims",
    "citizen-map",
    "citizen-propose-content",
    "admin-content",
)


def citizen_access_portal_types():
    """Return the portal types where the CitizenAccess is active"""
    portal_types = api.portal.get_tool("portal_types")
    types = []
    cls = cls_fullpath(ICitizenAccess)
    for portal_type in portal_types:
        if cls in getattr(portal_types[portal_type], "behaviors", []):
            types.append(portal_type)
    return types


def cls_fullpath(cls):
    """Return the complete path of a class"""
    return u"{module}.{classname}".format(module=cls.__module__, classname=cls.__name__)


class UnrestrictedUser(BaseUnrestrictedUser):
    """Unrestricted user that still has an id."""

    def getId(self):
        """Return the ID of the user."""
        return self.getUserName()


def execute_under_unrestricted_user(portal, function, user, *args, **kwargs):
    """
    Execute code under unrestricted user privileges.

    Example how to call::

        execute_under_unrestricted_user(
            portal,
            doSomeNormallyNotAllowedStuff,
            "admin",
            source_folder,
            target_folder,
        )

    @param portal: Reference to ISiteRoot object whose access controls we are using
    @param function: Method to be called with special privileges
    @param user: User to use for special privileges
    @param args: Passed to the function
    @param kwargs: Passed to the function
    """
    sm = getSecurityManager()
    try:
        try:
            # Create new temporary unrestricted user using the current user ID
            # Note that the username (getId()) is left in exception
            # tracebacks in the error_log,
            # so it is an important thing to store.
            tmp_user = UnrestrictedUser(user, "", [""], "")
            # Wrap the user in the acquisition context of the portal
            tmp_user = tmp_user.__of__(portal.acl_users)
            newSecurityManager(None, tmp_user)
            # Call the function
            return function(*args, **kwargs)
        except LinkIntegrityNotificationException as e:
            raise e
        except:  # noqa
            # If special exception handlers are needed, run them here
            pass
    finally:
        # Restore the old security manager
        setSecurityManager(sm)


def can_edit(user, context):
    """
    Verify if the given user have the 'Modify portal content' permission
    on the given context
    """
    if api.user.is_anonymous():
        return False
    if is_citizen(user):
        return False
    return api.user.has_permission(
        "Modify portal content", user=user, obj=context
    ) or api.user.has_permission("Copy or Move", user=user, obj=context)


def can_edit_citizen(user, context):
    """
    Verify if the given user have the 'Modify citizen content' permission
    on the given context
    """
    if api.user.is_anonymous():
        return False
    return api.user.has_permission("Modify citizen content", user=user, obj=context)


def is_citizen(user):
    """
    Verify if the given user is a citizen (member of the Citizens group)
    """
    if api.user.is_anonymous():
        return False
    user_groups = api.group.get_groups(user=user)
    # In some usecase, the group can be `None`
    return "Citizens" in [g.id for g in user_groups if g]


def can_claim(user, context):
    """Verify if the given user can claim the given context"""
    if can_edit_citizen(user, context) is True:
        return False
    if not is_citizen(user):
        return False
    return context.portal_type in get_allowed_claim_types()


def have_claimed(user, context):
    """Verify if the given user alredy claimed the given context"""
    return user.id in get_claim_users(context)


def get_claim_users(context):
    """Return the users that have claimed the given context"""
    claims = get_annotations(context).get("claim", [])
    return [isinstance(e, str) and e or e[0] for e in claims]


def get_draft_folder(context):
    """Return the citizen draft folder for the given context (object)"""
    navigation_root = api.portal.get_navigation_root(context)
    folder = navigation_root[DRAFT_FOLDER_ID]
    if context.portal_type not in allowed_content_types(folder):
        return context.aq_parent
    return folder


def allowed_content_types(context):
    """Return the ids of the allowed content types for the given context"""
    return [
        p.id
        for p in execute_under_unrestricted_user(
            context, context.allowedContentTypes, ""
        )
    ]


def dict_2_vocabulary(dictionary):
    """Transform a dictionary into a vocabulary"""
    terms = [SimpleVocabulary.createTerm(k, k, v) for k, v in dictionary.items()]
    return SimpleVocabulary(terms)


def get_settings():
    return getUtility(IRegistry).forInterface(ISettings)


def get_creation_folder(context, request, portal_type):
    """Return the creation folder for a proposal from a citizen"""
    navigation_root = api.portal.get_navigation_root(context)
    adapter = queryMultiAdapter(
        (context, request), ICitizenCreationFolder, name=portal_type
    )
    if adapter:
        return adapter.get_folder(navigation_root, portal_type)
    adapter = getMultiAdapter((context, request), ICitizenCreationFolder)
    return adapter.get_folder(navigation_root, portal_type)


def get_allowed_creation_types():
    settings = get_settings()
    allowed_types = settings.creation_types
    if not allowed_types:
        allowed_types = citizen_access_portal_types()
    return allowed_types


def get_allowed_claim_types():
    settings = get_settings()
    claim_types = settings.claim_types
    if not claim_types:
        claim_types = citizen_access_portal_types()
    return claim_types


def get_annotations(context):
    annotations = IAnnotations(context)
    if ANNOTATION_KEY not in annotations:
        annotations[ANNOTATION_KEY] = PersistentDict()
    return annotations[ANNOTATION_KEY]


def get_required_fields(portal_type):
    """Return the required fields for a given dexterity portal type"""
    schemas = utils.iterSchemataForType(portal_type)
    fields = []
    for schema in schemas:
        fields.extend(
            [(k, f) for k, f in field.Fields(schema).items() if f.field.required]
        )
    return fields


def iterate_claims(claims):
    """Backward compatibility method for claims without reason"""
    for claim in claims:
        if isinstance(claim, tuple):
            yield claim
        else:
            yield (claim, "")


def get_claims(context):
    annotations = get_annotations(context)
    if "claim" not in annotations:
        return []
    return iterate_claims(annotations["claim"])


def add_claim(context, user_id, reason):
    annotations = get_annotations(context)
    if "claim" not in annotations:
        annotations["claim"] = []
    if user_id not in [e[0] for e in iterate_claims(annotations["claim"])]:
        annotations["claim"].append((user_id, reason))
        annotations._p_changed = True
        context.reindexObject()


def remove_claim(context, user_id):
    annotations = get_annotations(context)
    annotations["claim"] = [
        e for e in iterate_claims(annotations["claim"]) if e[0] != user_id
    ]
    annotations._p_changed = True
    context.reindexObject()


def remove_relation(obj):
    catalog = getUtility(ICatalog)
    for relation in get_relations(obj):
        if relation.to_object is None:
            catalog.unindex(relation)


def get_working_copy(obj):
    try:
        return get_wc(obj)
    except KeyError:  # This happen if a link is broken
        return


def has_claim(obj):
    """Check if the given object has awaiting claims"""
    if obj != get_working_copy(obj):
        annotations = get_annotations(obj)
        return len(annotations.get("claim", [])) > 0
    return False


def validation_required(obj):
    """Check if the given object is awaiting for validation"""
    if obj == get_working_copy(obj):
        annotations = get_annotations(obj)
        return annotations.get("validation_required", False)
    return False



# -*- coding: utf-8 -*-

from Products.Five import BrowserView
from cpskin.citizen import security
from cpskin.citizen import utils
from plone import api
from plone.app.iterate.interfaces import IWorkingCopy


class CitizenSecurityMigrateView(BrowserView):
    def migrate(self):
        return security.ManageSecurity(self.request).migrate()


class CitizenFixBrokenLinks(BrowserView):
    def migrate(self):
        query = {
            "portal_type": utils.citizen_access_portal_types(),
            "provides": {"not": IWorkingCopy},
        }
        brains = [b for b in api.content.find(**query) if b.citizens]
        return self._migrate(brains)

    def _migrate(self, brains):
        result = []
        for brain in brains:
            obj = brain.getObject()
            try:
                utils.get_wc(obj)
            except KeyError as e:
                if "copy_of" in e.__str__():
                    utils.remove_relation(obj)
                    result.append(brain.getURL())
        return result



# -*- coding: utf-8 -*-
from cpskin.citizen.utils import is_citizen
from plone import api

import geocoder
import logging

logger = logging.getLogger("cpskin.citizen get lat lng")


def create_lat_lon(event):
    user = event.object
    if not is_citizen(user):
        return
    if user.getProperty("latitude", False):
        return
    geocode = get_lat_lng_from_address(
        user.getProperty("street"),
        user.getProperty("number"),
        user.getProperty("zip_code"),
        user.getProperty("location"),
    )
    # Use geocode.ok to avoid UnicodeDecodeError
    if getattr(geocode, "ok", False) is True:
        username = user.getUserName()
        member = api.user.get(username=username)
        member.setMemberProperties(mapping={"latitude": str(geocode.lat)})
        member.setMemberProperties(mapping={"longitude": str(geocode.lng)})


def get_lat_lng_from_address(street, number, zip_code, city):
    if street is None or city is None:
        return
    if street is not None and city is not None:
        address = "{} {} {} {}".format(number, street, zip_code, city)
        return geocoder.google(address)



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from z3c.table.table import Table
from zope.interface import implements

from cpskin.citizen.dashboard import interfaces


class DashboardTable(Table):
    implements(interfaces.IFacetedDashboardTable)

    cssClassEven = u"even"
    cssClassOdd = u"odd"
    cssClasses = {"table": "listing dashboard-listing"}

    batchSize = 20
    startBatchingAt = 30
    results = []

    _ignored_columns = (
        "Title",
        "select_row",
        "Creator",
        "getText",
        "ModificationDate",
        "CreationDate",
        "review_state",
        "actions",
        "pretty_link",
    )

    def setUpColumns(self):
        columns = super(DashboardTable, self).setUpColumns()
        return [c for c in columns if c.__name__ not in self._ignored_columns]


class CitizenContentTable(DashboardTable):
    implements(interfaces.IFacetedDashboardCitizenContentTable)


class CitizenClaimTable(DashboardTable):
    implements(interfaces.IFacetedDashboardCitizenClaimTable)


class AdminContentTable(DashboardTable):
    implements(interfaces.IFacetedDashboardAdminContentTable)


class AdminClaimTable(DashboardTable):
    implements(interfaces.IFacetedDashboardAdminClaimTable)


class CitizenMapTable(Table):
    implements(interfaces.IFacetedDashboardCitizenMapTable)

    batchSize = 20000



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from plone import api

from cpskin.citizen import _
from cpskin.citizen import utils


class DraftFolderLocator(object):

    title = _(u"Citizen draft folder")

    def __init__(self, context):
        self.context = context

    @property
    def available(self):
        """Ensure that the current user is a citizen"""
        current_user = api.user.get_current()
        return utils.can_edit_citizen(current_user, self.context)

    def __call__(self):
        return utils.get_draft_folder(self.context)



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from AccessControl.SecurityInfo import ModuleSecurityInfo
from Products.CMFCore.permissions import setDefaultRoles


security = ModuleSecurityInfo("cpskin.citizen")

security.declarePublic("ModifyCitizenContent")
ModifyCitizenContent = "Modify citizen content"
setDefaultRoles(ModifyCitizenContent, ("Manager", "Citizen Editor"))

security.declarePublic("AdminCitizenContent")
AdminCitizenContent = "Admin citizen content"
setDefaultRoles(
    AdminCitizenContent, ("Manager", "Site Administrator", "Reviewer", "Editor")
)



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from plone import api
from plone.app.iterate.browser.cancel import Cancel

from cpskin.citizen import utils


class CancelCitizenView(Cancel):
    def __call__(self):
        if "form.button.Cancel" in self.request.form:
            current_user = api.user.get_current()
            if utils.can_edit(current_user, self.context):
                return super(CancelCitizenView, self).__call__()
            else:
                return self._unrestricted_call(current_user)
        else:
            return super(CancelCitizenView, self).__call__()

    def _unrestricted_call(self, current_user):
        return utils.execute_under_unrestricted_user(
            api.portal.get(),
            super(CancelCitizenView, self).__call__,
            current_user.id,
        )



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from persistent.dict import PersistentDict
from plone import api
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import applyProfile
from plone.app.testing import login
from plone.app.testing import logout
from plone.testing import z2
from zope.annotation import IAnnotations

import transaction
import unittest

from cpskin.citizen import ANNOTATION_KEY
from cpskin.citizen.adapter import CitizenCreationFolderAdapter
from cpskin.citizen.browser.settings import ISettings

import cpskin.citizen


class CpskinCitizenLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        self.loadZCML(package=cpskin.citizen, name="testing.zcml")
        z2.installProduct(app, "imio.dashboard")
        z2.installProduct(app, "Products.DateRecurringIndex")

    def setUpPloneSite(self, portal):
        applyProfile(portal, "Products.CMFPlone:plone")
        applyProfile(portal, "cpskin.citizen:testing")

        manager = api.user.create(
            email="manager@manager.com", username="manager", password="manager"
        )
        api.user.grant_roles(user=manager, roles=("Manager",))

        citizen = api.user.create(
            email="citizen@citizen.com", username="citizen", password="citizen"
        )
        citizen.setMemberProperties(mapping={"fullname": u"Céline Dupont"})
        newcitizen = api.user.create(
            email="newcitizen@citizen.com", username="newcitizen", password="citizen"
        )
        newcitizen.setMemberProperties(mapping={"fullname": "Jean Dupont"})
        api.group.add_user(groupname="Citizens", user=citizen)
        api.group.add_user(groupname="Citizens", user=newcitizen)

        login(portal, "manager")
        api.content.create(
            type="Document",
            id="citizen-document",
            title="Citizen Document",
            container=portal,
            citizens=["citizen"],
        )
        api.content.create(
            type="Document",
            id="citizens-document",
            title="Citiznes Document éàç",
            container=portal,
            citizens=["citizen", "newcitizen"],
        )
        api.content.create(
            type="Document", id="document", title="Document", container=portal
        )
        document = api.content.create(
            type="Document",
            id="claim-document",
            title="Claim Document",
            container=portal,
        )
        annotations = IAnnotations(document)
        annotations[ANNOTATION_KEY] = PersistentDict()
        annotations[ANNOTATION_KEY]["claim"] = ["citizen"]

        obj = api.content.create(
            type="Folder", id="documents", title="Documents", container=portal
        )
        api.content.transition(obj=obj, transition="publish")

        api.portal.set_registry_record(
            name="creation_types", value=["Document"], interface=ISettings
        )

        logout()
        transaction.commit()

    def tearDownZope(self, app):
        """Tear down Zope."""
        z2.uninstallProduct(app, "imio.dashboard")
        z2.uninstallProduct(app, "Products.DateRecurringIndex")


CPSKIN_CITIZEN_FIXTURE = CpskinCitizenLayer()


CPSKIN_CITIZEN_INTEGRATION_TESTING = IntegrationTesting(
    bases=(CPSKIN_CITIZEN_FIXTURE,), name="CpskinCitizenLayer:IntegrationTesting"
)


CPSKIN_CITIZEN_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(CPSKIN_CITIZEN_FIXTURE,), name="CpskinCitizenLayer:FunctionalTesting"
)


CPSKIN_CITIZEN_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(CPSKIN_CITIZEN_FIXTURE, REMOTE_LIBRARY_BUNDLE_FIXTURE, z2.ZSERVER_FIXTURE),
    name="CpskinCitizenLayer:AcceptanceTesting",
)


class BaseTestCase(unittest.TestCase):
    @property
    def portal(self):
        return self.layer["portal"]

    @property
    def citizen_document(self):
        return self.portal["citizen-document"]

    @property
    def citizens_document(self):
        return self.portal["citizens-document"]

    @property
    def claim_document(self):
        return self.portal["claim-document"]

    @property
    def document(self):
        return self.portal["document"]

    @property
    def documents(self):
        return self.portal["documents"]

    @property
    def dashboards(self):
        return self.portal["citizen-dashboard"]

    @property
    def drafts(self):
        return self.portal["citizen-drafts"]


class TestCreationFolderAdapter(CitizenCreationFolderAdapter):
    def get_folder(self, navigation_root, portal_type):
        return navigation_root["documents"]



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from Acquisition import aq_inner
from plone.app.iterate.browser import checkin
from plone.app.iterate.interfaces import CheckinException
from zope.component import getMultiAdapter

from cpskin.citizen import utils


class CheckinView(checkin.Checkin):
    def __call__(self):
        context = aq_inner(self.context)

        if "form.button.Checkin" in self.request.form:
            control = getMultiAdapter((context, self.request), name=u"iterate_control")
            if not control.checkin_allowed():
                raise CheckinException(u"Not a checkout")
            annotations = utils.get_annotations(context)
            annotations["comment"] = None
            annotations["validation_required"] = False
        return super(CheckinView, self).__call__()



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.geo.geographer.interfaces import IGeoreferenced
from cpskin.citizen import utils
from cpskin.citizen.behavior import ICitizenAccess
from cpskin.citizen.utils import get_baseline
from cpskin.citizen.utils import get_working_copy
from plone.app.iterate.interfaces import IWorkingCopy
from plone.indexer import indexer
from zope.component import queryAdapter
from zope.interface import Interface


@indexer(ICitizenAccess)
def citizens_indexer(obj):
    if obj != get_working_copy(obj):
        return obj.citizens


@indexer(ICitizenAccess)
def validation_required_indexer(obj):
    return utils.validation_required(obj)


@indexer(ICitizenAccess)
def citizen_claim_indexer(obj):
    if obj != get_working_copy(obj):
        return utils.get_claim_users(obj)


@indexer(ICitizenAccess)
def is_draft(obj):
    return IWorkingCopy.providedBy(obj)


@indexer(ICitizenAccess)
def has_claim(obj):
    return utils.has_claim(obj)


@indexer(Interface)
def is_geolocated(obj):
    geo_adapter = queryAdapter(obj, IGeoreferenced)
    if geo_adapter:
        return geo_adapter.geo.get("coordinates") is not None
    return False


@indexer(ICitizenAccess)
def is_citizen_content(obj):
    if obj != get_working_copy(obj) and obj.citizens:
        return True
    try:
        baseline = get_baseline(obj)
        if getattr(baseline, "citizens", None):
            return True
    except KeyError:
        # This happen when a link is broken
        pass
    return utils.has_claim(obj)


@indexer(ICitizenAccess)
def citizen_action(obj):
    if utils.has_claim(obj) is True:
        return "allow-management"
    if utils.validation_required(obj) is True:
        return "awaiting-for-approval"



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from Acquisition import aq_inner
from Products.statusmessages.interfaces import IStatusMessage
from plone import api
from plone.app.iterate import PloneMessageFactory as PMF
from plone.app.iterate.browser import checkout
from plone.app.iterate.interfaces import CheckoutException
from plone.app.iterate.interfaces import ICheckinCheckoutPolicy
from zope.component import getMultiAdapter

from cpskin.citizen import utils


class CheckoutView(checkout.Checkout):
    def checkout(self, context):
        """Perform the checkout"""
        control = getMultiAdapter((context, self.request), name=u"iterate_control")
        if not control.checkout_allowed():
            raise CheckoutException(u"Not allowed")

        location = self.request.form.get("checkout_location", None)
        locator = None
        try:
            locator = [
                c["locator"] for c in self.containers() if c["name"] == location
            ][0]
        except IndexError:
            status_message = IStatusMessage(self.request)
            status_message.addStatusMessage(
                PMF("Cannot find checkout location"), type="stop"
            )
            view_url = context.restrictedTraverse("@@plone_context_state").view_url()
            self.request.response.redirect(view_url)
            return

        policy = ICheckinCheckoutPolicy(context)
        wc = policy.checkout(locator())

        # we do this for metadata update side affects which will update lock info
        context.reindexObject("review_state")

        current_user = api.user.get_current()
        if utils.can_edit(current_user, context):
            status_message = IStatusMessage(self.request)
            status_message.addStatusMessage(PMF("Check-out created"), type="info")
            view_url = wc.restrictedTraverse("@@plone_context_state").view_url()
        else:
            view_url = "{0}/@@edit-citizen".format(
                wc.restrictedTraverse("@@plone_context_state").view_url()
            )
        self.request.response.redirect(view_url)

    def __call__(self):
        context = aq_inner(self.context)

        containers = list(self.containers())
        if len(containers) == 1:
            # Special case for when there's only when folder to select
            self.request.form["form.button.Checkout"] = 1
            self.request.form["checkout_location"] = containers[0]["name"]

        # We want to redirect to a specific template, else we might
        # end up downloading a file
        if "form.button.Checkout" in self.request.form:
            self.checkout(context)
        elif "form.button.Cancel" in self.request.form:
            view_url = context.restrictedTraverse("@@plone_context_state").view_url()
            self.request.response.redirect(view_url)
        else:
            return self.index()



# -*- coding: utf-8 -*-

from cpskin.citizen import testing
from plone import api
from plone.app.testing import login
from plone.app.testing import logout
from plone.locking.interfaces import ILockable


class TestCancelView(testing.BaseTestCase):
    layer = testing.CPSKIN_CITIZEN_INTEGRATION_TESTING

    def setUp(self):
        login(self.portal, "manager")
        self.cancel_document = api.content.create(
            container=self.portal,
            type="Document",
            id="cancel-document",
            title="Cancel",
            citizens=["citizen"],
        )
        logout()
        self._create_checkout()

    def _create_checkout(self, user="citizen"):
        login(self.portal, user)
        view = self.cancel_document.restrictedTraverse("@@content-checkout")
        view.request.form["form.button.Checkout"] = "1"
        view.__call__()
        self.document_draft = self.drafts["cancel-document"]
        logout()

    def tearDown(self):
        login(self.portal, "manager")
        if "cancel-document" in self.drafts:
            api.content.delete(self.drafts["cancel-document"])
        lockable = ILockable(self.cancel_document)
        if lockable.locked():
            lockable.clear_locks()
        api.content.delete(self.cancel_document)
        logout()

    def test_cancel_as_manager_no_submit(self):
        login(self.portal, "manager")
        view = self.document_draft.restrictedTraverse("@@cancel-citizen")
        view.__call__()
        self.assertTrue("cancel-document" in self.drafts)

    def test_cancel_as_manager(self):
        login(self.portal, "manager")
        view = self.document_draft.restrictedTraverse("@@cancel-citizen")
        view.request.form["form.button.Cancel"] = "1"
        view.__call__()
        self.assertFalse("cancel-document" in self.drafts)

    def test_cancel_as_citizen_no_submit(self):
        login(self.portal, "citizen")
        view = self.document_draft.restrictedTraverse("@@cancel-citizen")
        view.__call__()
        self.assertTrue("cancel-document" in self.drafts)

    def test_cancel_as_citizen(self):
        login(self.portal, "citizen")
        view = self.document_draft.restrictedTraverse("@@cancel-citizen")
        view.request.form["form.button.Cancel"] = "1"
        view.__call__()
        self.assertFalse("cancel-document" in self.drafts)



# -*- coding: utf-8 -*-

from cpskin.citizen import testing
from cpskin.citizen import notification
from mock import patch
from plone import api


class TestNotification(testing.BaseTestCase):
    layer = testing.CPSKIN_CITIZEN_INTEGRATION_TESTING

    def setUp(self):
        self.citizen = api.user.get(userid="citizen")

    def tearDown(self):
        api.portal.set_registry_record(
            "cpskin.citizen.browser.settings.ISettings.manager_email", u""
        )

    def test_get_recipient(self):
        self.assertEqual("citizen@citizen.com", notification._get_recipient("citizen"))

    def test_get_missing_recipient(self):
        """ Test a special usecase where the citizen user does not exist anymore """
        self.assertIsNone(notification._get_recipient("foo"))

    def test_notify_content_validated_single_citizen(self):
        with patch.object(api.portal, "send_email", return_value=None) as mock:
            notification.notify_content_validated(self.citizen_document)
            self.assertTrue(mock.called)
            self.assertEqual(1, mock.call_count)

    def test_notify_content_validated_multi_citizen(self):
        with patch.object(api.portal, "send_email", return_value=None) as mock:
            notification.notify_content_validated(self.citizens_document)
            self.assertTrue(mock.called)
            self.assertEqual(2, mock.call_count)

    def test_notify_content_validated_no_citizen(self):
        with patch.object(api.portal, "send_email", return_value=None) as mock:
            notification.notify_content_validated(self.claim_document)
            self.assertFalse(mock.called)

    def test_notify_content_refused_single_citizen(self):
        with patch.object(api.portal, "send_email", return_value=None) as mock:
            notification.notify_content_refused(self.citizen_document)
            self.assertTrue(mock.called)
            self.assertEqual(1, mock.call_count)

    def test_notify_content_refused_multi_citizen(self):
        with patch.object(api.portal, "send_email", return_value=None) as mock:
            notification.notify_content_refused(self.citizens_document)
            self.assertTrue(mock.called)
            self.assertEqual(2, mock.call_count)

    def test_notify_content_refused_no_citizen(self):
        with patch.object(api.portal, "send_email", return_value=None) as mock:
            notification.notify_content_refused(self.claim_document)
            self.assertFalse(mock.called)

    def test_notify_content_awaiting_validation(self):
        with patch.object(api.portal, "send_email", return_value=None) as mock:
            notification.notify_content_awaiting_validation(
                self.citizens_document, self.citizen
            )
            self.assertTrue(mock.called)
            self.assertEqual(1, mock.call_count)

    def test_notify_content_awaiting_validation_with_recipients(self):
        api.portal.set_registry_record(
            "cpskin.citizen.browser.settings.ISettings.manager_email", u"foo@bar.com"
        )
        with patch.object(api.portal, "send_email", return_value=None) as mock:
            notification.notify_content_awaiting_validation(
                self.citizens_document, self.citizen
            )
            self.assertTrue(mock.called)
            self.assertEqual(2, mock.call_count)

    def test_notify_content_awaiting_access(self):
        with patch.object(api.portal, "send_email", return_value=None) as mock:
            notification.notify_content_awaiting_access(
                self.citizens_document, self.citizen
            )
            self.assertTrue(mock.called)
            self.assertEqual(1, mock.call_count)

    def test_notify_content_awaiting_access_with_recipients(self):
        api.portal.set_registry_record(
            "cpskin.citizen.browser.settings.ISettings.manager_email",
            u"foo@bar.com;bar@foo.com",
        )
        with patch.object(api.portal, "send_email", return_value=None) as mock:
            notification.notify_content_awaiting_access(
                self.citizens_document, self.citizen
            )
            self.assertTrue(mock.called)
            self.assertEqual(3, mock.call_count)



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""
from cpskin.citizen import _
from cpskin.citizen import security
from cpskin.citizen.dashboard.interfaces import IAdminDashboard
from cpskin.citizen.dashboard.interfaces import ICitizenDashboard
from cpskin.citizen.dashboard.interfaces import ICitizenMyContent
from cpskin.citizen.dashboard.portlet import DashboardPortletAssignment
from cpskin.citizen.interfaces import ICitizenContentSubMenu
from cpskin.citizen.interfaces import ICitizenDashboardFolder
from cpskin.citizen.interfaces import ICitizenProposeContentFolder
from eea.facetednavigation.layout.interfaces import IFacetedLayout
from plone import api
from plone.app.controlpanel.security import ISecuritySchema
from plone.portlets.constants import CONTEXT_CATEGORY
from plone.portlets.interfaces import ILocalPortletAssignmentManager
from plone.portlets.interfaces import IPortletAssignmentMapping
from plone.portlets.interfaces import IPortletManager
from zope.component import getAdapter
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.container.interfaces import INameChooser
from zope.globalrequest import getRequest
from zope.i18n import translate
from zope.interface import alsoProvides

import os


def post_install(context):
    """Post install script"""
    if context.readDataFile("cpskincitizen_default.txt") is None:
        return
    portal = api.portal.get()
    # Create the draft folder for each language
    lrfs = api.content.find(context=portal, portal_type="LRF")
    lrfs = [(e.getObject(), e.id) for e in lrfs]
    if not lrfs:
        lrfs = [
            (api.portal.get_navigation_root(portal), api.portal.get_default_language())
        ]
    dashboards = (
        (
            u"citizen-content",
            _(u"Citizen Content"),
            (ICitizenDashboard, ICitizenMyContent, ICitizenContentSubMenu),
            "dashboard/faceted_config.xml",
        ),
        (
            u"citizen-claims",
            _(u"Citizen Claims"),
            (ICitizenDashboard, ICitizenContentSubMenu),
            "dashboard/faceted_config.xml",
        ),
        (
            u"admin-content",
            _(u"Admin Citizen Content"),
            (IAdminDashboard,),
            "dashboard/faceted_admin_config.xml",
        ),
    )
    for folder, lng in lrfs:
        if u"citizen-drafts" not in folder:
            draft_folder = api.content.create(
                type="Folder",
                title=translate(_(u"Citizen Drafts"), target_language=lng),
                id=u"citizen-drafts",
                container=folder,
                exclude_from_nav=True,
            )
            api.content.disable_roles_acquisition(obj=draft_folder)
        if u"citizen-dashboard" not in folder:
            api.content.create(
                type="Folder",
                title=translate(_(u"Citizen Dashboard"), target_language=lng),
                id=u"citizen-dashboard",
                container=folder,
                exclude_from_nav=True,
            )
            add_portlet(
                folder[u"citizen-dashboard"], assignment=DashboardPortletAssignment()
            )

        dashboard_folder = folder[u"citizen-dashboard"]
        alsoProvides(dashboard_folder, ICitizenDashboardFolder)

        id = u"citizen-map"
        if id not in dashboard_folder:
            api.content.create(
                type="Folder",
                title=translate(_(u"Citizen Map"), target_language=lng),
                id=id,
                container=dashboard_folder,
                exclude_from_nav=True,
            )
            setup_faceted_dashboard_config(
                dashboard_folder[id],
                (ICitizenDashboard,),
                "faceted-map-view",
                "dashboard/faceted_map_view_config.xml",
            )

        id = u"citizen-propose-content"
        if id not in dashboard_folder:
            api.content.create(
                type="Folder",
                title=translate(_(u"Citizen propose"), target_language=lng),
                id=id,
                container=dashboard_folder,
                exclude_from_nav=True,
            )
        alsoProvides(dashboard_folder[id], ICitizenProposeContentFolder)
        alsoProvides(dashboard_folder[id], ICitizenContentSubMenu)

        for id, title, interfaces, template in dashboards:
            if id not in dashboard_folder:
                api.content.create(
                    type="Folder",
                    title=translate(title, target_language=lng),
                    id=id,
                    container=dashboard_folder,
                    exclude_from_nav=True,
                )
                setup_faceted_dashboard_config(
                    dashboard_folder[id], interfaces, config_path=template
                )

        id = u"my-profile"
        if id not in dashboard_folder:
            folder_url = folder.absolute_url()
            personal_url = u"{0}/@@personal-information".format(folder_url)
            api.content.create(
                type="Link",
                title=translate(_(u"My profile"), target_language=lng),
                id=id,
                remoteUrl=personal_url,
                container=dashboard_folder,
                exclude_from_nav=True,
            )

        disable_portlet_inheritance(folder[u"citizen-dashboard"])

    if not api.group.get(groupname="Citizens"):
        api.group.create(groupname="Citizens", title="Citizens")

    # set auto enable registration
    security_schema = getAdapter(api.portal.get(), ISecuritySchema)
    security_schema.enable_self_reg = True

    # add user_registration_fields
    portal_properties = api.portal.get_tool("portal_properties")
    site_properties = portal_properties.site_properties
    site_properties.user_registration_fields = (
        "fullname",
        "username",
        "email",
        "password",
        "mail_me",
        "street",
        "number",
        "zip_code",
        "location",
    )

    # Add extra versionable content types
    add_allowed_versionable_contenttype("organization")

    # Adapt security
    security.ManageSecurity(getRequest()).migrate()


def setup_faceted_dashboard_config(
    context,
    interfaces,
    layout="citizen-faceted-dashboard",
    config_path="dashboard/faceted_config.xml",
):
    subtyper = context.restrictedTraverse("@@faceted_subtyper")
    if subtyper.is_faceted:
        return
    subtyper.enable()
    for interface in interfaces:
        alsoProvides(context, interface)
    context.restrictedTraverse("@@faceted_settings").toggle_left_column()
    IFacetedLayout(context).update_layout(layout)
    config_xml = os.path.join(os.path.dirname(__file__), config_path)
    context.unrestrictedTraverse("@@faceted_exportimport").import_xml(
        import_file=open(config_xml, "r")
    )


def add_portlet(context, column_name="plone.leftcolumn", assignment=None):
    if not assignment:
        return
    column = getUtility(IPortletManager, column_name)
    manager = getMultiAdapter((context, column), IPortletAssignmentMapping)
    chooser = INameChooser(manager)
    manager[chooser.chooseName(None, assignment)] = assignment


def disable_portlet_inheritance(context, column_name="plone.leftcolumn"):
    column = getUtility(IPortletManager, column_name)
    manager = getMultiAdapter((context, column), ILocalPortletAssignmentManager)
    manager.setBlacklistStatus(CONTEXT_CATEGORY, True)


def add_allowed_versionable_contenttype(portal_type):
    portal_repository = api.portal.get_tool("portal_repository")
    types = portal_repository.getVersionableContentTypes()
    if portal_type not in types:
        types.append(portal_type)
    portal_repository.setVersionableContentTypes(types)



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from Products.Five import BrowserView
from plone import api

from cpskin.citizen import utils
from cpskin.citizen.behavior import ICitizenAccess


class CitizenControl(BrowserView):
    def __init__(self, *args, **kwargs):
        super(CitizenControl, self).__init__(*args, **kwargs)
        self.is_anonymous = api.user.is_anonymous()
        self.current_user = api.user.get_current()

    def is_citizen(self):
        """Check if the current user is a citizen"""
        if self.is_anonymous:
            return False
        return utils.is_citizen(self.current_user)

    def is_admin(self):
        """Check if the current user can administrate citizen contents"""
        if self.is_anonymous:
            return False
        allowed_roles = ["Editor", "Manager", "Reviewer", "Site Administrator"]
        user_roles = api.user.get_roles(user=self.current_user)
        return len([r for r in user_roles if r in allowed_roles]) > 0

    def is_citizen_content(self):
        """Check if the current content can be administrated by citizen"""
        return ICitizenAccess.providedBy(self.context)



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from plone import api
from plone.app.layout.viewlets import common as base
from plone.app.stagingbehavior.utils import get_working_copy

from cpskin.citizen import utils


class CitizenBaseViewlet(base.ViewletBase):
    def update(self):
        self.current_user = api.user.get_current()
        if self.can_view:
            self.annotations = utils.get_annotations(self.context)

    @property
    def can_view(self):
        return True

    @property
    def comment(self):
        return self.annotations.get("comment", None)

    @property
    def validation_required(self):
        return self.annotations.get("validation_required", False)

    @property
    def has_claims(self):
        return len(self.annotations.get("claim", [])) > 0

    @property
    def awaiting_claims(self):
        claims = []
        for claim in utils.get_claims(self.context):
            user_id, reason = claim
            user = api.user.get(userid=user_id)
            if user:
                claims.append(
                    {
                        "id": claim,
                        "user_id": user_id,
                        "username": user.getProperty("fullname"),
                        "email": user.getProperty("email"),
                        "reason": reason,
                    }
                )
        return claims


class CitizenEditionViewlet(CitizenBaseViewlet):
    def update(self):
        super(CitizenEditionViewlet, self).update()
        if self.can_view:
            self.have_claimed = utils.have_claimed(self.current_user, self.context)

    @property
    def can_view(self):
        if api.user.is_anonymous() is True:
            return False
        return utils.is_citizen(self.current_user)

    @property
    def can_edit(self):
        return utils.can_edit_citizen(self.current_user, self.context)

    @property
    def can_claim(self):
        return utils.can_claim(self.current_user, self.context)

    @property
    def can_cancel(self):
        if utils.can_edit_citizen(self.current_user, self.context) is False:
            return False
        return self.context == get_working_copy(self.context)

    @property
    def can_ask_validation(self):
        if self.context != get_working_copy(self.context):
            return False
        return not self.validation_required

    @property
    def show_fieldset(self):
        if self.can_edit or self.can_cancel or self.can_ask_validation:
            return True
        if self.can_claim and not self.have_claimed:
            return True
        return False


class CitizenAdminViewlet(CitizenBaseViewlet):
    pass


class CitizenProposeContentViewlet(CitizenBaseViewlet):
    pass


class CitizenInfoViewlet(CitizenBaseViewlet):
    @property
    def can_view(self):
        if api.user.is_anonymous() is True:
            return False
        return utils.is_citizen(self.current_user)

    @property
    def url(self):
        return "{0}/citizen-dashboard/".format(
            api.portal.get_navigation_root(self.context).absolute_url()
        )



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from Products.Five import BrowserView
from cpskin.citizen import _
from cpskin.citizen import utils
from cpskin.citizen import notification
from plone import api
from plone.z3cform.layout import FormWrapper
from z3c.form import button
from z3c.form.field import Fields
from z3c.form.form import Form
from zExceptions import Unauthorized
from zope import schema
from zope.interface import Interface


class ClaimSchema(Interface):
    reason = schema.Text(
        title=_(u"Explain briefly why you are asking the management of this content"),
        required=True,
    )


class ClaimForm(Form):
    fields = Fields(ClaimSchema)
    ignoreContext = True

    def update(self):
        self.current_user = api.user.get_current()
        super(ClaimForm, self).update()

    @button.buttonAndHandler(_(u"Confirm"), name="confirm")
    def handleConfirm(self, action):
        data, errors = self.extractData()
        if self.can_claim is False:
            raise Unauthorized("Cannot claim this content")
        if errors:
            self.status = self.formErrorsMessage
            return
        view_url = self.context.absolute_url()
        utils.add_claim(self.context, self.current_user.id, data["reason"])
        notification.notify_content_awaiting_access(self.context, self.current_user)
        self.request.response.redirect(view_url)

    @button.buttonAndHandler(_(u"Cancel"), name="cancel")
    def handleCancel(self, action):
        if self.can_claim is False:
            raise Unauthorized("Cannot claim this content")
        view_url = self.context.absolute_url()
        self.request.response.redirect(view_url)

    @property
    def can_claim(self):
        return utils.can_claim(self.current_user, self.context)


class ClaimCitizenView(FormWrapper):
    form = ClaimForm


class ClaimCitizenApprovalView(BrowserView):
    def __call__(self):
        self.claims = [e for e in utils.get_claims(self.context)]
        if not self._is_valid_request:
            return
        if self.confirm:
            if self.context.citizens is None:
                self.context.citizens = []
            if self.user not in self.context.citizens:
                self.context.citizens.append(self.user)
        utils.remove_claim(self.context, self.user)
        self.request.response.redirect(self.context.absolute_url())

    @property
    def _user_claims(self):
        return [e[0] for e in self.claims]

    @property
    def _is_valid_request(self):
        return (self.confirm or self.refuse) and self.user in self._user_claims

    @property
    def confirm(self):
        return "form.button.Confirm" in self.request.form

    @property
    def refuse(self):
        return "form.button.Refuse" in self.request.form

    @property
    def user(self):
        return self.request.form.get("userid", "")



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from plone import api

from cpskin.citizen import testing


class TestLocalRole(testing.BaseTestCase):
    layer = testing.CPSKIN_CITIZEN_INTEGRATION_TESTING

    def test_role_on_regular_document(self):
        """Test the local roles on a document with no citizens"""
        manager = api.user.get("manager")
        roles = api.user.get_roles(user=manager, obj=self.document)
        self.assertNotIn("Citizen Editor", roles)

        citizen = api.user.get("citizen")
        roles = api.user.get_roles(user=citizen, obj=self.document)
        self.assertNotIn("Citizen Editor", roles)

    def test_role_on_citizen_document(self):
        """Test the local roles on a document with citizens"""
        manager = api.user.get("manager")
        roles = api.user.get_roles(user=manager, obj=self.citizen_document)
        self.assertNotIn("Citizen Editor", roles)

        citizen = api.user.get("citizen")
        roles = api.user.get_roles(user=citizen, obj=self.citizen_document)
        self.assertIn("Citizen Editor", roles)



# -*- coding: utf-8 -*-

from cpskin.citizen import _
from cpskin.citizen import utils
from plone import api
from plone.protect.auto import safeWrite


def _get_draft_folders():
    portal = api.portal.get()
    lrfs = api.content.find(context=portal, portal_type="LRF")
    if not lrfs:
        return [portal[utils.DRAFT_FOLDER_ID]]
    return [lrf.getObject()[utils.DRAFT_FOLDER_ID] for lrf in lrfs]


def _get_dashboard_folders():
    portal = api.portal.get()
    lrfs = api.content.find(context=portal, portal_type="LRF")
    folders = []
    if not lrfs:
        base_folder = portal[utils.DASHBOARD_FOLDER_ID]
        folders.append(base_folder)
        folders.extend([base_folder[e] for e in utils.DASHBOARD_FOLDERS_IDS])
        return folders
    for lrf in lrfs:
        base_folder = lrf.getObject()[utils.DASHBOARD_FOLDER_ID]
        folders.append(base_folder)
        folders.extend([base_folder[e] for e in utils.DASHBOARD_FOLDERS_IDS])
    return folders


class ManageSecurity(object):
    def __init__(self, request):
        self.request = request
        self.report = {}
        self.draft_folders = _get_draft_folders()
        self.dashboard_folders = _get_dashboard_folders()

    def add_report_msg(self, obj, msg, type="error"):
        url = obj.absolute_url()
        if url not in self.report:
            self.report[url] = []
        self.report[url].append((type, msg))
        return self.report

    def _manage_role_acquisition(self, obj, block=True):
        """ Manage local role acquisition for a contentish object """
        if block is True:
            expected_value = 1
            me = api.content.disable_roles_acquisition
            msg = (
                _("Role acquisition was disabled"),
                _("Role acquisition already disabled"),
            )
        else:
            expected_value = 0
            me = api.content.enable_roles_acquisition
            msg = (
                _("Role acquisition was enabled"),
                _("Role acquisition already enabled"),
            )
        if getattr(obj, "__ac_local_roles_block__", 0) != expected_value:
            safeWrite(obj, self.request)
            me(obj=obj)
            self.add_report_msg(obj, msg[0], type="error")
        else:
            self.add_report_msg(obj, msg[1], type="info")

    def _manage_local_roles(self, obj, roles):
        """ Manage local roles for an object """
        for user, current_roles in obj.get_local_roles():
            if user not in roles.keys():
                safeWrite(obj, self.request)
                obj.manage_delLocalRoles([user])
                self.add_report_msg(
                    obj, _("Remove local roles for user {0}").format(user), type="error"
                )
        current_local_roles = {k: v for k, v in obj.get_local_roles()}
        for user, new_roles in roles.items():
            if user not in current_local_roles.keys():
                safeWrite(obj, self.request)
                obj.manage_setLocalRoles(user, new_roles)
                self.add_report_msg(
                    obj, _("Add local roles for user {0}").format(user), type="error"
                )
            else:
                if new_roles != current_local_roles[user]:
                    safeWrite(obj, self.request)
                    obj.manage_setLocalRoles(user, new_roles)
                    self.add_report_msg(
                        obj,
                        _("update local roles for user {0}").format(user),
                        type="error",
                    )
                else:
                    self.add_report_msg(
                        obj,
                        _("local roles are correct for user {0}").format(user),
                        type="info",
                    )

    def migrate(self):
        self._migrate_draft_folders()
        self._migrate_dashboard_folders()
        return self.report

    def _migrate_draft_folders(self):
        # Fix permissions on folder (No citizen users must have access to draft folder)
        # Do not acquire from parent folder
        for draft_folder in self.draft_folders:
            # role acquisition must be disabled
            self._manage_role_acquisition(draft_folder, block=True)
            self._manage_local_roles(draft_folder, roles={"admin": ("Owner",)})

    def _migrate_dashboard_folders(self):
        # Ensure that permissions on dashboard folders are corrects
        # Root : citizen can view / do not acquire from parent folder
        # Mes contenus, citizen-map : citizen -> can view
        # citizen-content, citizen-claims, citizen-propose-content : inherit from parent folder (citizen -> can view)
        # admin-claims, admin-content : citizen can not view
        security_map = {
            "citizen-dashboard": {
                "roles": {"admin": ("Owner",), "Citizens": ("Reader",)},
                "block": True,
            },
            "citizen-content": {"roles": {"admin": ("Owner",)}, "block": False},
            "citizen-claims": {"roles": {"admin": ("Owner",)}, "block": False},
            "citizen-map": {"roles": {"admin": ("Owner",)}, "block": False},
            "citizen-propose-content": {"roles": {"admin": ("Owner",)}, "block": False},
            "admin-content": {"roles": {"admin": ("Owner",)}, "block": True},
            "admin-claims": {"roles": {"admin": ("Owner",)}, "block": True},
        }
        for dashboard_folder in self.dashboard_folders:
            security = security_map.get(dashboard_folder.id)
            self._manage_role_acquisition(dashboard_folder, block=security["block"])
            self._manage_local_roles(dashboard_folder, security["roles"])



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from Products.CMFCore.exceptions import AccessControl_Unauthorized
from plone import api
from plone.app.testing import login
from plone.app.testing import logout
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
from zope.publisher.browser import TestRequest

from cpskin.citizen import testing
from cpskin.citizen import utils
from cpskin.citizen.browser.settings import ISettings


class TestUtils(testing.BaseTestCase):
    layer = testing.CPSKIN_CITIZEN_INTEGRATION_TESTING

    def setUp(self):
        self.request = TestRequest()

    def tearDown(self):
        api.portal.set_registry_record(
            name="creation_types", value=["Document"], interface=ISettings
        )

    def test_citizen_access_portal_types(self):
        """Test 'citizen_access_portal_types' function"""
        types = utils.citizen_access_portal_types()
        self.assertListEqual(
            sorted(["Document", "News Item", "Event", "organization"]), sorted(types)
        )

    def test_cls_fullpath(self):
        """Test 'cls_fullpath' function"""
        from cpskin.citizen.interfaces import ICpskinCitizenLayer

        self.assertEqual(
            u"cpskin.citizen.interfaces.ICpskinCitizenLayer",
            utils.cls_fullpath(ICpskinCitizenLayer),
        )

    def test_can_edit_as_anonymous(self):
        """Ensure that anonymous user doesn't have edit permissions"""
        logout()
        self.assertTrue(api.user.is_anonymous())
        user = api.user.get_current()
        self.assertFalse(utils.can_edit(user, self.citizen_document))

    def test_can_edit_as_manager(self):
        """Ensure that manager users have edit permissions"""
        user = api.user.get("manager")
        self.assertTrue(utils.can_edit(user, self.citizen_document))
        logout()

    def test_can_edit_as_citizen(self):
        """Ensure that citizen users doesn't have edit permissions"""
        user = api.user.get("citizen")
        self.assertFalse(utils.can_edit(user, self.citizen_document))
        logout()

    def test_execute_under_unrestricted_user(self):
        """Test the 'execute_under_unrestricted_user' function"""
        login(self.portal, "citizen")
        kwargs = {
            "type": "Document",
            "container": self.portal,
            "id": "test-document",
            "title": "Test Document",
        }
        self.assertRaises(AccessControl_Unauthorized, api.content.create, **kwargs)
        self.assertIsNone(self.portal.get("test-document"))
        utils.execute_under_unrestricted_user(
            self.portal, api.content.create, "citizen", **kwargs
        )
        self.assertIsNotNone(self.portal.get("test-document"))

    def test_can_edit_citizen_as_anonymous(self):
        """Ensure that an anonymous user cannot edit citizen content"""
        logout()
        self.assertTrue(api.user.is_anonymous())
        user = api.user.get_current()
        self.assertFalse(utils.can_edit_citizen(user, self.citizen_document))

    def test_can_edit_citizen_as_manager(self):
        """Ensure that a manager user can edit citizen content"""
        user = api.user.get("manager")
        self.assertTrue(utils.can_edit_citizen(user, self.citizen_document))

    def test_can_edit_citizen_as_citizen(self):
        """
        Ensure that a citizen user can edit a content that he have access and
        cannot edit content that he doesn't have access
        """
        user = api.user.get("citizen")
        self.assertTrue(utils.can_edit_citizen(user, self.citizen_document))
        self.assertFalse(utils.can_edit_citizen(user, self.document))

    def test_is_citizen(self):
        """Test 'is_citizen' function"""
        manager = api.user.get("manager")
        self.assertFalse(utils.is_citizen(manager))
        citizen = api.user.get("citizen")
        self.assertTrue(utils.is_citizen(citizen))
        logout()
        anonymous = api.user.get_current()
        self.assertFalse(utils.is_citizen(anonymous))

    def test_can_claim(self):
        """Test 'can_claim' function"""
        user = api.user.get("citizen")
        self.assertTrue(utils.can_claim(user, self.document))
        self.assertFalse(utils.can_claim(user, self.citizen_document))

    def test_have_claimed(self):
        """Test 'have_claimed' function"""
        user = api.user.get("citizen")
        self.assertTrue(utils.have_claimed(user, self.claim_document))
        self.assertFalse(utils.have_claimed(user, self.document))

    def test_get_claim_user(self):
        """Test 'get_claim_users' function"""
        self.assertEqual(["citizen"], utils.get_claim_users(self.claim_document))
        self.assertEqual([], utils.get_claim_users(self.document))

    def test_get_draft_folder(self):
        """Test 'get_draft_folder' function"""
        self.assertEqual(
            self.portal["citizen-drafts"], utils.get_draft_folder(self.document)
        )

    def test_get_settings(self):
        """Test 'get_settings' function"""
        settings = utils.get_settings()
        self.assertEqual(["Document"], settings.creation_types)

    def test_get_creation_folder_default(self):
        """Test 'get_creation_folder' function default"""
        self.assertEqual(
            self.portal, utils.get_creation_folder(self.portal, self.request, "foo")
        )

    def test_get_creation_folder_custom_adapter(self):
        """Test 'get_creation_folder' function for a custom adapter"""
        self.assertEqual(
            self.portal["documents"],
            utils.get_creation_folder(self.portal, self.request, "Document"),
        )

    def test_get_annotations(self):
        """Test 'get_annotations' function"""
        annotations = utils.get_annotations(self.claim_document)
        self.assertEqual(["citizen"], annotations["claim"])

    def test_get_allowed_creation_types(self):
        self.assertEqual(["Document"], utils.get_allowed_creation_types())
        registry = getUtility(IRegistry).forInterface(ISettings)
        registry.creation_types = None
        registry_values = api.portal.get_registry_record(
            name="creation_types", interface=ISettings
        )
        self.assertEqual(None, registry_values)
        self.assertListEqual(
            sorted(["Document", "News Item", "Event", "organization"]),
            sorted(utils.get_allowed_creation_types()),
        )



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from plone import api


def afterMemberAdd(self, member, id, password, properties):
    """
    Called by portal_registration.addMember() after a member
    has been added successfully
    """
    if not api.group.get(groupname="Citizens"):
        return
    # if this is an auto registration, not an administrator action
    if api.user.is_anonymous():
        api.group.add_user(groupname="Citizens", user=member)



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from plone import api
from plone.principalsource.source import PrincipalSource
from zope.globalrequest import getRequest
from zope.i18n import translate
from zope.interface import implements
from zope.schema.interfaces import IContextSourceBinder

from cpskin.citizen import _
from cpskin.citizen import utils


class CitizensSource(PrincipalSource):
    def __init__(self, context):
        super(CitizensSource, self).__init__(context)

    def _search(self, id=None, exact_match=True):
        users = api.user.get_users(groupname="Citizens")
        if id is not None:
            return [{"id": u.id} for u in users if u.id == id]
        return [{"id": u.id} for u in users]


class CitizensSourceBinder(object):
    """Bind the principal source with either users or groups"""

    implements(IContextSourceBinder)

    def __call__(self, context):
        return CitizensSource(context)


CitizensVocabulary = CitizensSourceBinder()


class CitizensPortalTypesVocabularyFactory(object):
    def __call__(self, context):
        portal_types = api.portal.get_tool("portal_types")
        content = {
            k: v.title
            for k, v in portal_types.items()
            if k in utils.citizen_access_portal_types()
        }
        return utils.dict_2_vocabulary(content)


CitizensPortalTypesVocabulary = CitizensPortalTypesVocabularyFactory()


class CitizensAllowedCreationTypesVocabularyFactory(object):
    def _translate(self, msgid):
        request = getRequest()
        translation = translate(msgid, context=request)
        return translation.encode("utf-8")

    def __call__(self, context):
        portal_types = api.portal.get_tool("portal_types")
        content = {}
        for k, v in portal_types.items():
            if k not in utils.get_allowed_creation_types():
                continue
            title = self._translate(_(v.title))
            if v.description:
                description = self._translate(_(v.description))
                content[k] = "{0}  ({1})".format(title, description)
            else:
                content[k] = title
        return utils.dict_2_vocabulary(content)


CitizensAllowedCreationTypesVocabulary = CitizensAllowedCreationTypesVocabularyFactory()


class CitizensActionsVocabularyFactory(object):
    def __call__(self, context):
        return utils.dict_2_vocabulary({
            u"allow-management": _(u"Allow management"),
            u"awaiting-for-approval": _(u"Awaiting for approval"),
        })


CitizensActionsVocabulary = CitizensActionsVocabularyFactory()



# -*- coding: utf-8 -*-
from collective.geo.leaflet.interfaces import IGeoMap
from collective.geo.leaflet.geomap import GeoMap as GM
from plone import api
from zope.interface import implementer


@implementer(IGeoMap)
class GeoMap(GM):
    def map_center(self):
        user = api.user.get_current()
        latitude = user.getProperty("latitude", False)
        longitude = user.getProperty("longitude", False)
        if latitude and longitude:
            return {"longitude": str(longitude), "latitude": str(latitude)}
        else:
            return super(GeoMap, self).map_center()

    def geo_settings(self):
        settings = super(GeoMap, self).geo_settings()
        # Update zoom map level
        settings["zoom"] = str(16)
        return settings



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from borg.localrole.interfaces import ILocalRoleProvider
from zope.interface import implements


class LocalRoleAdapter(object):
    implements(ILocalRoleProvider)

    def __init__(self, context):
        self.context = context

    def getRoles(self, principal):
        """Grant permission for principal"""
        citizens = getattr(self.context, "citizens", []) or []
        if principal in citizens:
            return ("Reader", "Citizen Editor")
        return []

    def getAllRoles(self):
        """Grant permissions"""
        if not getattr(self.context, "citizens", None):
            yield ("", ("",))
            raise StopIteration
        else:
            permissions = ("Reader", "Citizen Editor")
            for citizen in self.context.citizens:
                yield (citizen, permissions)



# -*- coding: utf-8 -*-

from Products.CMFPlone.utils import safe_unicode
from cpskin.citizen import _
from cpskin.citizen.browser.settings import ISettings
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from plone import api
from smtplib import SMTPException
from zope.i18n import translate


def _generate_email(body, obj):
    html = safe_unicode(
        """<html><body style="font-family: arial;">{0},<br><br>""" "{1}</body></html>"
    )
    html_body = html.format(translate(_("Hello"), context=obj.REQUEST), body)
    msg = MIMEMultipart("alternative")
    msg.attach(MIMEText(html_body.encode("utf-8"), "html", "UTF-8"))
    return msg


def _get_recipient(userid):
    """ Return the email address for the given userid """
    user = api.user.get(userid=userid)
    if user:
        return user.getProperty("email")


def _get_admin_recipients():
    record = api.portal.get_registry_record(
        interface=ISettings, name="manager_email", default=""
    )
    if not record:
        return []
    return record.split(";")


def _send_email(recipient, sender, subject, body):
    """ Wrapper for plone.api.send_email function that handle special usecases """
    try:
        api.portal.send_email(
            recipient=recipient, sender=sender, subject=safe_unicode(subject), body=body
        )
    except SMTPException:
        pass


def notify_content_validated(obj):
    """ Send an email to notify the citizen owner(s) of the document that the content
    was published
    """
    citizens = getattr(obj, "citizens", [])
    if not citizens:
        # This can happen when this is not a citizen manage content
        return
    portal = api.portal.get()
    title = translate(
        _(u"You have a message about a content that you manage on {portal_url}"),
        context=obj.REQUEST,
    )
    title = title.format(portal_url=portal.absolute_url())
    body = translate(
        _(
            u"""The content "{title}" that you manage on the website {portal_url}"""
            u"""<a href="{link}" alt="{title}">requires a modification from you</a>"""
        ),
        context=obj.REQUEST,
    )
    body = body.format(
        link=obj.absolute_url(), title=obj.title, portal_url=portal.absolute_url()
    )

    for citizen in citizens:
        recipient = _get_recipient(citizen)
        _send_email(recipient, portal.email_from_address, title, _generate_email(body, obj))


def notify_content_refused(obj):
    """ Send an email to notify the citizen owner(s) of the document that a change
    must be made
    """
    citizens = getattr(obj, "citizens", [])
    if not citizens:
        # This can happen when this is not a citizen manage content
        return
    portal = api.portal.get()
    title = translate(
        _(u"You have a message about a content that you manage on {portal_url}"),
        context=obj.REQUEST,
    )
    title = title.format(portal_url=portal.absolute_url())
    body = translate(
        _(
            u"""The content "{title}" that you manage on the website {portal_url}"""
            u"""<a href="{link}" alt="{title}">was published online</a>"""
        ),
        context=obj.REQUEST,
    )
    body = body.format(
        link=obj.absolute_url(), title=obj.title, portal_url=portal.absolute_url()
    )

    for citizen in citizens:
        recipient = _get_recipient(citizen)
        _send_email(recipient, portal.email_from_address, title, _generate_email(body, obj))


def notify_content_awaiting_validation(obj, user):
    """ Send an email to notify the administrator that a new content
    is awaiting for review
    """
    portal = api.portal.get()
    title = translate(
        _(u"You have a request about a citizen content on {portal_url}"),
        context=obj.REQUEST,
    )
    title = title.format(portal_url=portal.absolute_url())
    body = translate(
        _(
            u"""<a href="{link}" alt="{title}">A review for a citizen content</a>"""
            u"was requested by {name}"
        ),
        context=obj.REQUEST,
    )
    body = body.format(
        link=obj.absolute_url(),
        title=obj.title,
        name=user.getProperty("fullname").decode("utf8"),
    )

    recipients = [portal.email_from_address]
    recipients.extend(_get_admin_recipients())
    for recipient in recipients:
        _send_email(recipient, portal.email_from_address, title, _generate_email(body, obj))


def notify_content_awaiting_access(obj, user):
    """ Send an email to notify the administrator that an access to a content
    is awaiting for approval
    """
    portal = api.portal.get()
    title = translate(
        _(u"You have a request about a citizen content on {portal_url}"),
        context=obj.REQUEST,
    )
    title = title.format(portal_url=portal.absolute_url())
    body = translate(
        _(
            u"""<a href="{link}" alt="{title}">A new access request</a> from {name} """
            u"for a citizen content is awaiting for approval"
        ),
        context=obj.REQUEST,
    )
    body = body.format(
        link=obj.absolute_url(),
        title=obj.title,
        name=user.getProperty("fullname").decode("utf8"),
    )

    recipients = [portal.email_from_address]
    recipients.extend(_get_admin_recipients())
    for recipient in recipients:
        _send_email(recipient, portal.email_from_address, title, _generate_email(body, obj))



# -*- coding: utf-8 -*-

from plone import api
from plone.app.stagingbehavior.utils import get_working_copy


def citizen_content_removed(obj, event):
    draft = get_working_copy(obj)
    if draft:
        api.content.remove(obj)



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from plone import api
from zope.interface import implements

from cpskin.citizen.dashboard.interfaces import ICitizenDashboardQuery


class DefaultQueryFilters(object):
    implements(ICitizenDashboardQuery)

    def apply_filters(self, query):
        query["is_draft"] = False
        query["object_provides"] = u"cpskin.citizen.behavior.ICitizenAccess"
        return query


class AdminContentQueryFilters(object):
    implements(ICitizenDashboardQuery)

    def apply_filters(self, query):
        query["object_provides"] = u"cpskin.citizen.behavior.ICitizenAccess"
        query["is_citizen_content"] = True
        return query


class CitizenContentQueryFilters(DefaultQueryFilters):
    def apply_filters(self, query):
        current_user = api.user.get_current()
        query = super(CitizenContentQueryFilters, self).apply_filters(query)
        query["citizens"] = current_user.id
        return query


class CitizenClaimQueryFilters(DefaultQueryFilters):
    def apply_filters(self, query):
        current_user = api.user.get_current()
        query = super(CitizenClaimQueryFilters, self).apply_filters(query)
        query["citizen_claim"] = current_user.id
        return query


class CitizenMapQueryFilters(DefaultQueryFilters):
    def apply_filters(self, query):
        citizen_map_portal_types = api.portal.get_registry_record(
            "cpskin.citizen.browser.settings.ISettings.map_view_types"
        )
        if citizen_map_portal_types and "portal_type" not in query:
            query["portal_type"] = citizen_map_portal_types
        query["is_geolocated"] = True
        return query



# -*- coding: utf-8 -*-

from plone import api
from plone.registry import Record
from plone.registry import field
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
from eea.facetednavigation.layout.interfaces import IFacetedLayout

import os


def upgrade_faceted_dashboard_config(
    context, layout=None, config_path="dashboard/faceted_config.xml"
):
    context.restrictedTraverse("@@faceted_settings").toggle_left_column()
    if layout is not None:
        IFacetedLayout(context).update_layout(layout)
    config_xml = os.path.join(os.path.dirname(__file__), config_path)
    context.unrestrictedTraverse("@@faceted_exportimport").import_xml(
        import_file=open(config_xml, "r")
    )


def upgrade_citizen_draft_folder(context):
    portal = api.portal.get()
    # Create the draft folder for each language
    lrfs = api.content.find(context=portal, portal_type="LRF")
    lrfs = [e.getObject() for e in lrfs]
    if not lrfs:
        lrfs = [api.portal.get_navigation_root(portal)]
    for folder in lrfs:
        if u"citizen-drafts" in folder:
            api.content.disable_roles_acquisition(obj=folder["citizen-drafts"])


def upgrade_to_1002(context):
    """
    - Add cpskin.citizen.browser.settings.ISettings.published_states record
    - Add is_citizen_content index
    - Add citizen_action index
    """
    setup_tool = api.portal.get_tool(name="portal_setup")
    setup_tool.runImportStepFromProfile("profile-cpskin.citizen:default", "catalog")

    brains = api.content.find(object_provides=u"cpskin.citizen.behavior.ICitizenAccess")
    for brain in brains:
        if brain.has_claim or brain.validation_required:
            brain.getObject().reindexObject()

    registry = getUtility(IRegistry)
    records = registry.records

    if "cpskin.citizen.browser.settings.ISettings.published_states" in records:
        return

    record = Record(
        field.List(
            title=u"Published states",
            description=u"List of object states that can be assimilate to published",
            value_type=field.Choice(
                title=u"states", vocabulary="plone.app.vocabularies.WorkflowStates"
            ),
        ),
        value=[
            u"active",
            u"published",
            u"published_and_shown",
            u"published_and_hidden",
        ],
    )
    records["cpskin.citizen.browser.settings.ISettings.published_states"] = record


def upgrade_to_1003(context):
    portal = api.portal.get()
    lrfs = api.content.find(context=portal, portal_type="LRF")
    lrfs = [(e.getObject(), e.id) for e in lrfs]
    if not lrfs:
        lrfs = [
            (api.portal.get_navigation_root(portal), api.portal.get_default_language())
        ]
    for folder, lng in lrfs:
        if u"citizen-dashboard" not in folder:
            # This happen if cpskin.citizen was not correctly configured
            continue
        dashboard_folder = folder[u"citizen-dashboard"]
        if "admin-claims" in dashboard_folder:
            api.content.delete(dashboard_folder["admin-claims"])
        if "admin-content" in dashboard_folder:
            upgrade_faceted_dashboard_config(
                dashboard_folder["admin-content"],
                config_path="dashboard/faceted_admin_config.xml",
            )
    setup_tool = api.portal.get_tool(name="portal_setup")
    setup_tool.runImportStepFromProfile("profile-cpskin.citizen:default", "actions")


def upgrade_to_1004(context):
    """
    - Add cpskin.citizen.browser.settings.ISettings.manager_email record
    """
    registry = getUtility(IRegistry)
    records = registry.records

    if "cpskin.citizen.browser.settings.ISettings.manager_email" in records:
        return

    record = Record(
        field.TextLine(
            title=u"Email address of persons that manage citizen contents",
            description=(
                u"If there are multiple email addresses, separate them with semicolons"
            ),
        ),
        value=u"",
    )
    records["cpskin.citizen.browser.settings.ISettings.manager_email"] = record



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from eea.facetednavigation.browser.app.query import FacetedQueryHandler
from zope.component import getUtility
from zope.component import queryUtility

from cpskin.citizen.dashboard.interfaces import ICitizenDashboardQuery


class DashboardQueryHandler(FacetedQueryHandler):
    def criteria(self, *args, **kwargs):
        query = super(DashboardQueryHandler, self).criteria(*args, **kwargs)
        utility = queryUtility(ICitizenDashboardQuery, name=self.context.id)
        if utility is None:
            utility = getUtility(ICitizenDashboardQuery, name="default")
        return utility.apply_filters(query)

    def table_view(self):
        view_name = "@@{0}-table-view".format(self.context.id)
        return self.context.restrictedTraverse(view_name)



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.z3cform.select2.widget.widget import MultiSelect2FieldWidget
from plone.app.stagingbehavior.interfaces import IStagingSupport
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel.directives import fieldset
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider

from cpskin.citizen import _


@provider(IFormFieldProvider)
class ICitizenAccess(IStagingSupport):

    fieldset("citizen_access", label=_(u"Citizen access"), fields=["citizens"])

    form.widget(citizens=MultiSelect2FieldWidget)
    citizens = schema.List(
        title=_(u"Citizens users"),
        value_type=schema.Choice(
            title=_(u"Citizens users"), vocabulary="cpskin.citizen.citizens"
        ),
        required=False,
    )


@implementer(ICitizenAccess)
@adapter(IDexterityContent)
class CitizenAccess(object):
    def __init__(self, context):
        self.context = context

    @property
    def citizens(self):
        return getattr(self.context, "citizens", [])

    @citizens.setter
    def citizens(self, value):
        self.context.citizens = value



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.z3cform.select2.widget.widget import MultiSelect2FieldWidget
from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper
from plone.app.registry.browser.controlpanel import RegistryEditForm
from plone.autoform import directives as form
from plone.z3cform import layout
from zope import schema
from zope.interface import Interface

from cpskin.citizen import _


class ISettings(Interface):

    proposal_folders = schema.Dict(
        title=_(u"Proposal default folders"),
        description=_(
            u"The folders where (by default) new content proposed "
            u"by citizens will be created"
        ),
        key_type=schema.Choice(
            title=_(u"Content type"),
            vocabulary="cpskin.citizen.portal_types",
            required=True,
        ),
        value_type=schema.TextLine(
            title=_(u"Default folder"),
            description=_(
                u"Path to the folder from the navigation root "
                u"(should not contains the language folder)"
            ),
            required=True,
        ),
        required=False,
    )

    form.widget(claim_types=MultiSelect2FieldWidget)
    claim_types = schema.List(
        title=_(u"Claim content types"),
        description=_(u"Allowed content types for the claim procedure"),
        value_type=schema.Choice(
            title=_(u"Claim content types"), vocabulary="cpskin.citizen.portal_types"
        ),
        required=False,
    )

    form.widget(creation_types=MultiSelect2FieldWidget)
    creation_types = schema.List(
        title=_(u"Creation content types"),
        description=_(
            u"Allowed content types for creation by citizens "
            u"(by default all content types are allowed)"
        ),
        value_type=schema.Choice(
            title=_(u"Claim content types"), vocabulary="cpskin.citizen.portal_types"
        ),
        required=False,
    )

    form.widget(creation_types=MultiSelect2FieldWidget)
    map_view_types = schema.List(
        title=_(u"Map view content types"),
        description=_(
            u"Visibled content types for map view for citizens "
            u"(by default no content types are allowed)"
        ),
        value_type=schema.Choice(
            title=_(u"Map view content types"),
            vocabulary="plone.app.vocabularies.PortalTypes",
        ),
        required=False,
    )

    form.widget(creation_types=MultiSelect2FieldWidget)
    published_states = schema.List(
        title=_(u"Published states"),
        description=_(u"List of object states that can be assimilate to published"),
        value_type=schema.Choice(
            title=_(u"states"),
            vocabulary="plone.app.vocabularies.WorkflowStates",
        ),
        required=False,
    )

    manager_email = schema.TextLine(
        title=_(u"Email address of persons that manage citizen contents"),
        description=_(
            u"If there are multiple email addresses, separate them with semicolons"
        ),
    )


class SettingsEditForm(RegistryEditForm):
    schema = ISettings
    label = u"CPSkin Citizen settings"


SettingsView = layout.wrap_form(SettingsEditForm, ControlPanelFormWrapper)



# -*- coding: utf-8 -*-

from zope.publisher.browser import TestRequest
from plone import api
from cpskin.citizen import testing
from zope.component import getMultiAdapter
from cpskin.citizen.interfaces import ICitizenCreationFolder
from mock import patch


class TestAdapter(testing.BaseTestCase):
    layer = testing.CPSKIN_CITIZEN_INTEGRATION_TESTING

    def setUp(self):
        self.request = TestRequest()

    def test_creation_folder_adapter_default(self):
        """ Test CitizenCreationFolderAdapter """
        context = api.portal.get()
        adapter = getMultiAdapter((context, self.request), ICitizenCreationFolder)
        self.assertEqual(context, adapter.get_folder(context, "Document"))

    def test_creation_folder_adapter_root(self):
        """ Test CitizenCreationFolderAdapter """
        from cpskin.citizen import utils

        context = api.portal.get()
        adapter = getMultiAdapter((context, self.request), ICitizenCreationFolder)
        settings = type(
            "settings", (object,), {"proposal_folders": {"Document": "/"}}
        )()
        with patch.object(utils, "get_settings", return_value=settings):
            self.assertEqual(context, adapter.get_folder(context, "Document"))

    def test_creation_folder_adapter_folder(self):
        """ Test CitizenCreationFolderAdapter """
        from cpskin.citizen import utils

        context = api.portal.get()
        adapter = getMultiAdapter((context, self.request), ICitizenCreationFolder)
        settings = type(
            "settings", (object,), {"proposal_folders": {"Document": "documents"}}
        )()
        with patch.object(utils, "get_settings", return_value=settings):
            self.assertEqual(
                context["documents"], adapter.get_folder(context, "Document")
            )

        # with leading /
        settings = type(
            "settings", (object,), {"proposal_folders": {"Document": "/documents"}}
        )()
        with patch.object(utils, "get_settings", return_value=settings):
            self.assertEqual(
                context["documents"], adapter.get_folder(context, "Document")
            )

    def test_creation_folder_adapter_unknown_folder(self):
        """ Test CitizenCreationFolderAdapter """
        from cpskin.citizen import utils

        context = api.portal.get()
        adapter = getMultiAdapter((context, self.request), ICitizenCreationFolder)
        settings = type(
            "settings", (object,), {"proposal_folders": {"Document": "foo"}}
        )()
        with patch.object(utils, "get_settings", return_value=settings):
            self.assertRaisesRegexp(
                KeyError,
                "unknown creation folder: .*",
                adapter.get_folder,
                context,
                "Document",
            )



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from Products.Five import BrowserView
from cpskin.citizen import utils
from cpskin.citizen import notification
from plone import api
from zExceptions import Unauthorized


class ValidationRequiredCitizenView(BrowserView):
    def __call__(self):
        self.current_user = api.user.get_current()
        if self.can_edit_citizen is False:
            raise Unauthorized("Cannot require validation for this content")
        view_url = self.context.absolute_url()
        if "form.button.Confirm" in self.request.form:
            annotations = utils.get_annotations(self.context)
            annotations["validation_required"] = True
            self.context.reindexObject()
            notification.notify_content_awaiting_validation(
                self.context, self.current_user
            )
            self.request.response.redirect(view_url)
        elif "form.button.Cancel" in self.request.form:
            self.request.response.redirect(view_url)
        return self.index()

    @property
    def can_edit_citizen(self):
        return utils.can_edit_citizen(self.current_user, self.context)



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from datetime import datetime
from plone import api
from plone.z3cform.layout import FormWrapper
from z3c.form import button
from z3c.form import field
from z3c.form.browser.radio import RadioFieldWidget
from z3c.form.form import Form
from zExceptions import Unauthorized
from zope import schema
from zope.interface import Interface
from zope.schema.vocabulary import getVocabularyRegistry

from cpskin.citizen import _
from cpskin.citizen import utils


class IProposeCitizenContent(Interface):

    type = schema.Choice(
        title=_(u"Portal Type"),
        vocabulary="cpskin.citizen.allowed_creation_portal_types",
        required=True,
    )

    title = schema.TextLine(title=_(u"Title"), required=True)


class ProposeCitizenForm(Form):
    fields = field.Fields(IProposeCitizenContent)
    action = "@@propose-citizen"
    ignoreContext = True
    fields["type"].widgetFactory = RadioFieldWidget

    @button.buttonAndHandler(_(u"Confirm"))
    def handleApply(self, action):
        self.current_user = api.user.get_current()
        data, errors = self.extractData()
        if errors or self.is_citizen is False:
            raise Unauthorized(u"Cannot propose a content")
        container = utils.get_creation_folder(
            self.context, self.request, data.get("type")
        )
        fields = self.get_required_fields(data.get("type"))
        vr = getVocabularyRegistry()
        default_values = {}
        for k, f in fields:
            if isinstance(f.field, schema.Datetime):
                default_values[k] = datetime.now()
            elif isinstance(f.field, schema.Choice):
                voc = vr.get(container, f.field.vocabularyName)
                default_values[k] = voc.by_value.values()[0].value
        data.update(default_values)
        content = self.create_original(container, **data)
        self.create_draft(content)
        return

    @property
    def is_citizen(self):
        if api.user.is_anonymous() is True:
            return False
        return utils.is_citizen(self.current_user)

    def create_original(self, container, **kwargs):
        content = utils.execute_under_unrestricted_user(
            api.portal.get(), api.content.create, "admin", container=container, **kwargs
        )
        content.citizens = [self.current_user.id]
        return content

    def create_draft(self, content):
        url = "{0}/@@edit-citizen".format(content.absolute_url())
        self.request.response.redirect(url)

    def get_required_fields(self, portal_type):
        ignored_fields = ("title",)

        fields = utils.get_required_fields(portal_type)
        fields = [
            (k, f)
            for k, f in fields
            if f.field.default is None and k not in ignored_fields
        ]
        return fields


class ProposeCitizenView(FormWrapper):
    form = ProposeCitizenForm

    def links(self):
        """Return the creation folders links"""
        folders = [
            utils.get_creation_folder(self.context, self.request, e)
            for e in utils.get_allowed_creation_types()
        ]
        return [{"href": p.absolute_url(), "title": p.Title()} for p in folders]



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from Acquisition import aq_inner
from plone import api
from plone.app.stagingbehavior.browser import control

from cpskin.citizen import utils
from cpskin.citizen.behavior import ICitizenAccess


class DefaultControl(control.Control):
    def return_allowed(self):
        return False


class Control(control.Control):
    def checkout_allowed(self):
        """ Check if a checkout is allowed """
        context = aq_inner(self.context)
        if super(Control, self).checkout_allowed() is False:
            return False
        if ICitizenAccess.providedBy(context):
            current_user = api.user.get_current()
            if utils.is_citizen(current_user):
                return utils.can_edit_citizen(current_user, context)
        return True

    def cancel_allowed(self):
        """Check if a checkout cancel is allowed"""
        context = aq_inner(self.context)
        if ICitizenAccess.providedBy(context):
            current_user = api.user.get_current()
            if utils.is_citizen(current_user):
                return utils.can_edit_citizen(current_user, context)
        return self.return_allowed()

    def return_allowed(self):
        """Check if a return checkout is allowed"""
        return super(Control, self).cancel_allowed()



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory

from cpskin.citizen import testing


class TestVocabularies(testing.BaseTestCase):
    layer = testing.CPSKIN_CITIZEN_INTEGRATION_TESTING

    def _get_vocabulary_values(self, name):
        vocabulary = getUtility(IVocabularyFactory, name=name)(self.portal)
        return [t.value for t in vocabulary]

    def test_citizen_vocabulary(self):
        """Test 'CitizenVocabulary'"""
        voc = self._get_vocabulary_values("cpskin.citizen.citizens")
        self.assertListEqual(sorted(["citizen", "newcitizen"]), sorted(voc))

    def test_citizen_portal_types_vocabulary(self):
        """Test 'CitizensPortalTypesVocabulary'"""
        voc = self._get_vocabulary_values("cpskin.citizen.portal_types")
        self.assertListEqual(
            sorted(["Document", "News Item", "Event", "organization"]), sorted(voc)
        )

    def test_citizen_creation_types_vocabulary(self):
        """Test 'CitizensAllowedCreationTypesVocabulary'"""
        voc = self._get_vocabulary_values(
            "cpskin.citizen.allowed_creation_portal_types"
        )
        self.assertListEqual(sorted(["Document"]), sorted(voc))



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from plone import api
from plone.app.iterate import archiver
from zExceptions import Unauthorized

from cpskin.citizen import utils


class ContentArchiverAdapter(archiver.ContentArchiver):
    def save(self, checkin_message):
        current_user = api.user.get_current()
        if utils.can_edit(current_user, self.context):
            return self.repository.save(self.context, checkin_message)
        if utils.can_edit_citizen(current_user, self.context):
            return utils.execute_under_unrestricted_user(
                api.portal.get(),
                self.repository.save,
                current_user.id,
                self.context,
                checkin_message,
            )
        raise Unauthorized(u"Cannot edit this content")



# -*- coding: utf-8 -*-

from cpskin.citizen import testing
from cpskin.citizen import utils
from plone import api
from plone.app.testing import login
from plone.app.testing import logout
from plone.locking.interfaces import ILockable
from zope.component import queryAdapter


class TestCheckout(testing.BaseTestCase):
    layer = testing.CPSKIN_CITIZEN_INTEGRATION_TESTING

    def setUp(self):
        login(self.portal, "manager")
        self.doc = api.content.create(
            container=self.portal,
            type="Document",
            id="test-document",
            title="doc",
            citizens=["citizen"],
        )
        self.event = api.content.create(
            container=self.portal,
            type="Event",
            id="test-event",
            title="event",
            citizens=["citizen"],
        )
        self.directory = api.content.create(
            container=self.portal, type="directory", id="directory", title="directory"
        )
        self.organization = api.content.create(
            container=self.directory,
            type="organization",
            id="test-organization",
            title="organization",
            citizens=["citizen"],
        )
        logout()

    def _create_checkout(self, obj, user="citizen"):
        login(self.portal, user)
        view = obj.restrictedTraverse("@@content-checkout")
        view.request.form["form.button.Checkout"] = "1"
        view.__call__()
        logout()

    def _cancel_checkout(self, obj, user="citizen"):
        login(self.portal, user)
        view = obj.restrictedTraverse("@@cancel-citizen")
        view.request.form["form.button.Cancel"] = "1"
        view.__call__()
        logout()

    def tearDown(self):
        login(self.portal, "manager")
        for obj_id in ["test-event", "test-document", "directory"]:
            if obj_id in self.drafts:
                api.content.delete(self.drafts[obj_id])
            lockable = queryAdapter(self.portal[obj_id], ILockable)
            if lockable and lockable.locked():
                lockable.clear_locks()
            api.content.delete(self.portal[obj_id])
        logout()

    def test_checkout_document(self):
        login(self.portal, "citizen")
        self._create_checkout(self.portal["test-document"])
        self.assertTrue("test-document" in self.drafts)

        self.assertEqual(
            self.portal["test-document"],
            utils.get_baseline(self.drafts["test-document"]),
        )
        self.assertEqual(
            self.portal["test-document"],
            utils.get_baseline(self.portal["test-document"]),
        )

        self.assertEqual(
            self.drafts["test-document"],
            utils.get_working_copy(self.portal["test-document"]),
        )
        self.assertEqual(
            self.drafts["test-document"],
            utils.get_working_copy(self.drafts["test-document"]),
        )

    def test_checkout_event(self):
        login(self.portal, "citizen")
        self._create_checkout(self.portal["test-event"])
        self.assertTrue("test-event" in self.drafts)

        self.assertEqual(
            self.portal["test-event"], utils.get_baseline(self.drafts["test-event"])
        )
        self.assertEqual(
            self.portal["test-event"], utils.get_baseline(self.portal["test-event"])
        )

        self.assertEqual(
            self.drafts["test-event"], utils.get_working_copy(self.portal["test-event"])
        )
        self.assertEqual(
            self.drafts["test-event"], utils.get_working_copy(self.drafts["test-event"])
        )

    def test_checkout_organization(self):
        login(self.portal, "citizen")
        self._create_checkout(self.organization)
        self.assertTrue("copy_of_test-organization" in self.directory)

        self.assertEqual(
            self.organization,
            utils.get_baseline(self.directory["copy_of_test-organization"]),
        )
        self.assertEqual(
            self.organization,
            utils.get_baseline(self.organization),
        )

        self.assertEqual(
            self.directory["copy_of_test-organization"],
            utils.get_working_copy(self.organization),
        )
        self.assertEqual(
            self.directory["copy_of_test-organization"],
            utils.get_working_copy(self.directory["copy_of_test-organization"]),
        )

    def test_cancel_checkout_document(self):
        login(self.portal, "citizen")
        self._create_checkout(self.portal["test-document"])
        self.assertTrue("test-document" in self.drafts)
        self._cancel_checkout(self.drafts["test-document"])
        self.assertTrue("test-document" not in self.drafts)
        self.assertIsNone(utils.get_working_copy(self.portal["test-document"]))
        self.assertIsNone(utils.get_baseline(self.portal["test-document"]))

    def test_cancel_checkout_event(self):
        login(self.portal, "citizen")
        self._create_checkout(self.portal["test-event"])
        self.assertTrue("test-event" in self.drafts)
        self._cancel_checkout(self.drafts["test-event"])
        self.assertTrue("test-event" not in self.drafts)
        self.assertIsNone(utils.get_working_copy(self.portal["test-event"]))
        self.assertIsNone(utils.get_baseline(self.portal["test-event"]))

    def test_cancel_checkout_organization(self):
        login(self.portal, "citizen")
        self._create_checkout(self.organization)
        self.assertTrue("copy_of_test-organization" in self.directory)
        self._cancel_checkout(self.directory["copy_of_test-organization"])
        self.assertTrue("copy_of_test-organization" not in self.directory)
        self.assertIsNone(utils.get_working_copy(self.organization))
        self.assertIsNone(utils.get_baseline(self.organization))



# -*- coding: utf-8 -*-
"""
cpskin.citizen
--------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from plone import api
from plone.app.stagingbehavior import copier
from zExceptions import Unauthorized

from cpskin.citizen import utils


class ContentCopierAdapter(copier.ContentCopier):
    def copyTo(self, container):
        current_user = api.user.get_current()
        if utils.can_edit(current_user, self.context):
            return super(ContentCopierAdapter, self).copyTo(container)
        if utils.can_edit_citizen(current_user, self.context):
            elements = utils.execute_under_unrestricted_user(
                api.portal.get(),
                super(ContentCopierAdapter, self).copyTo,
                "admin",
                container,
            )
            elements[0].creators = (current_user.id,)
            return elements
        raise Unauthorized(u"Cannot edit this content")
